@@ -190,17 +190,23 @@ int calculate_max_bcast_block(brgemm_desc_t *brg, const int adj_ld_block2) {
                       || brg->brgattr.max_bottom_vpad > 0)
             && brg->zp_type_a != brgemm_broadcast_t::none;
     const int beta_regs = !one_of(brg->beta, 1.f, 0.f);
+    const int b_vnni_regs = brg->is_f16_b_non_amx_vnni() ? 2 : 0;
 
     const int max_isa_regs = isa_num_vregs(brg->isa_impl);
     // note: the 'adj_ld_block2' already removes the necessary registers
     // for 'embd_bcst'
     auto max_reg_count = max_isa_regs - max_bcst_regs - beta_regs
-            - req_compensation - req_zp_a_comp_pads;
+            - req_compensation - req_zp_a_comp_pads - b_vnni_regs;
     if (req_zp_a_comp_pads)
         max_reg_count
                 = nstl::min(max_reg_count, max_isa_regs - max_bcst_regs - 5);
 
-    int max_bcast_block = max_reg_count - adj_ld_block2;
+    const int postops_regs = brg->attr()
+            ? injector::aux_vec_count(
+                    brg->attr()->post_ops_, brg->isa_impl, true)
+            : 0;
+    int max_bcast_block
+            = max_reg_count - nstl::max(adj_ld_block2, postops_regs);
 
     if (brg->is_bf16_emu) {
         // in theory, vmm bf16_emu register indices overlap with other vmm
@@ -234,20 +240,27 @@ status_t brgemm_blocking(brgemm_desc_t *brg) {
         brg->ldb = brg->load_dim / brg->ld_block;
         brg->ldb_tail = brg->load_dim % brg->ld_block;
 
-        int adj_ld_block2 = calculate_ldb_params(brg, 4);
-        int max_bcast_block = calculate_max_bcast_block(brg, adj_ld_block2);
-
-        // reduce 'ld_block2' to allow a larger 'bd_block'
         const int max_vpad = nstl::max(
                 brg->brgattr.max_top_vpad, brg->brgattr.max_bottom_vpad);
+
+        int adj_ld_block2 = calculate_ldb_params(brg, 4);
+        int max_bcast_block = calculate_max_bcast_block(brg, adj_ld_block2);
+        // reduce 'ld_block2' to allow a larger 'bd_block'
         if (is_superset(brg->isa_impl, avx2) && max_bcast_block < max_vpad) {
-            adj_ld_block2 = calculate_ldb_params(brg, 2);
-            max_bcast_block = calculate_max_bcast_block(brg, adj_ld_block2);
+            for (int try_ld_block2 = 2; try_ld_block2 > 0; --try_ld_block2) {
+                adj_ld_block2 = calculate_ldb_params(brg, try_ld_block2);
+                max_bcast_block = calculate_max_bcast_block(brg, adj_ld_block2);
+                if (max_bcast_block >= max_vpad) break;
+            }
+            // bcast block in brgemm kernel should be greater than virtual
+            // padding to avoid possible functional issues
+            if (max_bcast_block < max_vpad) return status::unimplemented;
         }
 
-        const int min_block = 1;
+        const int min_block = nstl::max(1, max_vpad);
+
         float best_bd_block_eff = 0.f;
-        brg->bd_block = 1;
+        brg->bd_block = max_bcast_block;
         for (int bd_block = max_bcast_block; bd_block >= min_block;
                 bd_block--) {
             const auto bd_block_disb = static_cast<float>(brg->bcast_dim)
@@ -770,8 +783,14 @@ status_t brdgmm_blocking(brgemm_desc_t *brg) {
     const int compute_vregs
             = jit_brdgmm_kernel_base_t<Xbyak::Zmm>::get_compute_vmm_count(*brg);
     const int bf16_emu_vregs = brg->is_bf16_emu * 4;
-    const int max_acc_vmms
-            = max_vregs - nstl::max(compute_vregs + aux_vregs, bf16_emu_vregs);
+    const int postops_regs = brg->attr()
+            ? injector::aux_vec_count(
+                    brg->attr()->post_ops_, brg->isa_impl, true)
+            : 0;
+
+    const int max_acc_vmms = max_vregs
+            - nstl::max(postops_regs,
+                    nstl::max(compute_vregs + aux_vregs, bf16_emu_vregs));
 
     if (brg->brgattr.hint_bs_group > 1) {
         // Check if we can actually apply bs grouping
