@@ -424,7 +424,8 @@ void emit_reorder_1d_tile(ngen::HW hw, GeneratorT *host,
             int esize = step;
             auto s = src.subregister(i, esize, src_stride);
             auto d = dst.subregister(i, esize, 1);
-            if (src_stride > 1 && s.getByteOffset() > 1) {
+            if ((src_stride > 1 && s.getByteOffset() > 0)
+                    || (d.getByteOffset() > 0 && dst_stride != src_stride)) {
                 host->mov(esize,
                         tmp1.subregister(0, ngen::DataType::ub)(src_stride),
                         s.reinterpret(0, ngen::DataType::ub)(src_stride));
@@ -545,7 +546,7 @@ void emit_reorder_1d_tile(ngen::HW hw, GeneratorT *host,
                     s.reinterpret(0, ngen::DataType::uw)(1), -0x40);
             // check for zero mantissa.
             host->and_(esize | host->nz | host->f1[0], host->null.uw(),
-                    s.reinterpret(0, ngen::DataType::uw)(1), 0x3FF);
+                    s.reinterpret(0, ngen::DataType::uw)(1), 0x00FF);
             host->eshr(esize, s.reinterpret(0, ngen::DataType::uw)(1),
                     s.reinterpret(0, ngen::DataType::uw)(1), 7);
             host->add(esize | host->f1[0],
@@ -1307,7 +1308,7 @@ public:
                                 x_stride, y_sub, y_stride);
                     });
             prev_layout = next_layout;
-            prev_rd = next_rd;
+            prev_rd = std::move(next_rd);
         }
     }
 
