@@ -24,10 +24,10 @@
 #include "common/gemm_utils.hpp"
 #include "common/utils.hpp"
 #include "gpu/intel/compute/kernel.hpp"
+#include "gpu/intel/compute/zero_pool.hpp"
 #include "gpu/intel/gemm/gpu_gemm.hpp"
 #include "gpu/intel/jit/gemm/gen_gemm_kernel.hpp"
 #include "gpu/intel/jit/gemm/jit_gemm_pd.hpp"
-#include "gpu/intel/jit/gemm/zero_pool.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -42,7 +42,7 @@ struct gen_gemm_t : public gpu_gemm_t {
 
         DECLARE_COMMON_PD_T("jit:gemm:any", gen_gemm_t);
 
-        status_t init(engine_t *engine) {
+        status_t init(impl::engine_t *engine) {
             using namespace prop_kind;
             using namespace data_type;
             using namespace primitive_kind;
@@ -65,10 +65,16 @@ struct gen_gemm_t : public gpu_gemm_t {
             int stepping = dev_info_->stepping_id();
 
             const auto d = desc();
-            wei_decomp_ = (utils::one_of(d->c_type(), f32, f16, bf16)
+            wei_decomp_ = (utils::one_of(d->c_type(), f32, f16, bf16, f8_e5m2,
+                                   f8_e4m3)
                                   && utils::one_of(d->a_type(), u8, s8, s4, u4)
-                                  && utils::one_of(d->b_type(), f16, f32, bf16))
+                                  && utils::one_of(d->b_type(), f16, f32, bf16,
+                                          f8_e5m2, f8_e4m3))
                     && attr()->mayiconvert(d->a_type(), f32);
+            dy_quant_enabled_ = (utils::one_of(d->c_type(), f32, f16, bf16)
+                    && utils::one_of(d->a_type(), u8, s8, s4, u4)
+                    && utils::one_of(d->b_type(), u8, s8));
+            quant_enabled_ = wei_decomp_ || dy_quant_enabled_;
             CHECK(set_default_formats(false));
 
             // If m = 1, swap A/B to use more efficient n = 1 kernels if possible.
@@ -106,7 +112,7 @@ struct gen_gemm_t : public gpu_gemm_t {
                 eff_ldb_ = utils::rnd_up(eff_ldb_, 16);
             }
 
-            if (wei_decomp_) {
+            if (quant_enabled_) {
                 attr_skip_mask |= smask_t::fpmath_mode
                         | smask_t::scales_runtime_data_type
                         | smask_t::scales_runtime_groups
@@ -114,9 +120,11 @@ struct gen_gemm_t : public gpu_gemm_t {
                         | smask_t::zero_points_runtime_groups;
             }
 
-            bool wei_zp = false, wei_zp_2d = false;
+            bool wei_zp_2d = false;
             auto wei_scales_type = data_type::undef;
+            auto src_scales_type = data_type::undef;
             int wei_q2d_group_k = 0;
+            int src_q2d_group_k = 0;
 
             // Check parameters.
             if (utils::one_of(d->c_type(), s32, f16, f32, u8, s8)
@@ -141,7 +149,10 @@ struct gen_gemm_t : public gpu_gemm_t {
                 VDISPATCH_GEMM(utils::one_of(d->a_type(), f64, f32, f16,
                                        f8_e5m2, f8_e4m3),
                         VERBOSE_UNSUPPORTED_DT);
-                VDISPATCH_GEMM(d->b_type() == d->a_type(),
+                VDISPATCH_GEMM(d->b_type() == d->a_type()
+                                || (utils::one_of(d->a_type(), f8_e5m2, f8_e4m3)
+                                        && utils::one_of(
+                                                d->b_type(), f8_e5m2, f8_e4m3)),
                         VERBOSE_INCONSISTENT_DT, "a", "b");
                 VDISPATCH_GEMM(utils::one_of(d->acc_type, d->a_type(), f32),
                         VERBOSE_UNSUPPORTED_DT);
@@ -158,17 +169,16 @@ struct gen_gemm_t : public gpu_gemm_t {
 
             VDISPATCH_GEMM(!has_blocks(), VERBOSE_BLOCKING_FAIL, "");
             VDISPATCH_GEMM(
-                    batch_dims() <= 2, VERBOSE_BAD_DIM, "batch", batch_dims());
+                    batch_dims() <= 4, VERBOSE_BAD_DIM, "batch", batch_dims());
             VDISPATCH_GEMM(
                     !utils::one_of(DNNL_RUNTIME_DIM_VAL, d->m(), d->n(), d->k(),
                             d->lda(), d->ldb(), d->ldc(), d->batch()),
                     VERBOSE_RUNTIMEDIM_UNSUPPORTED);
-            VDISPATCH_GEMM(
-                    IMPLICATION(with_bias(),
-                            utils::one_of(d->bias_type(), f64, f32, bf16, f16,
-                                    f8_e5m2, f8_e4m3)
-                                    && (d->bias_desc.ndims <= 3)
-                                    && utils::one_of(bias_cmask(), 0, 1, 2, 3)),
+            VDISPATCH_GEMM(IMPLICATION(with_bias(),
+                                   utils::one_of(d->bias_type(), f64, f32, bf16,
+                                           f16, f8_e5m2, f8_e4m3)
+                                           && (d->bias_desc.ndims <= 6)
+                                           && d->bias_mask() < 8),
                     VERBOSE_UNSUPPORTED_BIAS_CFG);
             VDISPATCH_GEMM(
                     IMPLICATION(with_bias(),
@@ -198,8 +208,7 @@ struct gen_gemm_t : public gpu_gemm_t {
                 CHECK(attr_zps.get(DNNL_ARG_B, &cmask_b));
                 CHECK(attr_zps.get(DNNL_ARG_C, &cmask_c));
 
-                wei_zp = a_zp;
-                wei_zp_2d = wei_decomp_ && (cmask_a == ((1 << 0) | (1 << 1)));
+                wei_zp_2d = attr_zps.get_groups_ndims(DNNL_ARG_A) > 1;
                 VDISPATCH_GEMM(
                         (utils::one_of(cmask_a, 0, 1 << 1, 1 << 2) || wei_zp_2d)
                                 && utils::one_of(cmask_b, 0, 1 << 0)
@@ -216,32 +225,67 @@ struct gen_gemm_t : public gpu_gemm_t {
                             = attr_zps.get_groups_ndims(DNNL_ARG_WEIGHTS) > 0
                             ? attr_zps.get_groups(DNNL_ARG_WEIGHTS)[0]
                             : 1;
+                    const auto wei_q2d_group_n
+                            = attr_zps.get_groups_ndims(DNNL_ARG_WEIGHTS) > 0
+                            ? attr_zps.get_groups(DNNL_ARG_WEIGHTS)[1]
+                            : 1;
+                    // Non-trivial N group unsupported.
+                    VDISPATCH_GEMM(
+                            wei_q2d_group_n == 1, VERBOSE_UNSUPPORTED_ZP_CFG);
                 }
+                // Zero points with non-trivial groups only supported when
+                // target tensor is being dequantized.
+                VDISPATCH_GEMM(
+                        IMPLICATION(wei_zp_2d,
+                                !dy_quant_enabled_
+                                        || utils::one_of(d->a_type(), s4, u4)
+                                        || wei_q2d_group_k == desc()->k()),
+                        VERBOSE_UNSUPPORTED_ZP_CFG);
             }
 
-            if (wei_decomp_
-                    && attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_
-                            == ((1 << 0) | (1 << 1)))
-                wei_scales_2d_ = true;
+            auto &wei_scales = attr()->scales_.get(DNNL_ARG_WEIGHTS);
+            auto &src_scales = attr()->scales_.get(DNNL_ARG_SRC);
+
+            if (quant_enabled_ && wei_scales.ndims_ > 1) wei_scales_2d_ = true;
+            if (quant_enabled_ && src_scales.ndims_ > 1) src_scales_2d_ = true;
 
             for (auto s : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
                 auto mask = attr()->scales_.get(s).mask_;
                 VDISPATCH_GEMM(utils::one_of(mask, 0, 1 << 0, 1 << 1, 1 << 2)
-                                || (s == DNNL_ARG_WEIGHTS && wei_scales_2d_),
+                                || (s == DNNL_ARG_WEIGHTS && wei_scales_2d_)
+                                || (s == DNNL_ARG_SRC && src_scales_2d_),
                         VERBOSE_UNSUPPORTED_SCALES_CFG);
             }
 
             if (wei_scales_2d_) {
-                VDISPATCH_GEMM(!(wei_zp && (ao_dims_ == 1 || bo_dims_ == 1)),
-                        VERBOSE_UNSUPPORTED_ZP_CFG);
-                auto &wei_scales = attr()->scales_.get(DNNL_ARG_WEIGHTS);
-                wei_scales_type = wei_scales.data_type_;
                 auto scales_group_k
                         = wei_scales.ndims_ > 0 ? wei_scales.group_dims_[0] : 1;
-                if (!wei_zp_2d)
-                    wei_q2d_group_k = scales_group_k;
+                if (scales_group_k >= d->k()) {
+                    wei_scales_2d_ = false;
+                } else {
+                    wei_scales_type = wei_scales.data_type_;
+                    if (!wei_zp_2d)
+                        wei_q2d_group_k = scales_group_k;
+                    else {
+                        VDISPATCH_GEMM((wei_q2d_group_k == scales_group_k),
+                                VERBOSE_UNSUPPORTED_SCALES_CFG);
+                    }
+                }
+                // Non-trivial N group unsupported.
+                VDISPATCH_GEMM(wei_scales.group_dims_[1] == 1,
+                        VERBOSE_UNSUPPORTED_ZP_CFG);
+            }
+            if (src_scales_2d_) {
+                src_scales_type = src_scales.data_type_;
+                src_po_sc_ = src_scales.mask_ == 2;
+                auto scales_group_k
+                        = src_scales.ndims_ > 0 ? src_scales.group_dims_[1] : 1;
+                if (scales_group_k >= d->k())
+                    src_scales_2d_ = false;
                 else {
-                    VDISPATCH_GEMM((wei_q2d_group_k == scales_group_k),
+                    src_q2d_group_k = scales_group_k;
+                    VDISPATCH_GEMM(dy_quant_enabled_
+                                    && utils::one_of(eff_a_type(), s4, u4),
                             VERBOSE_UNSUPPORTED_SCALES_CFG);
                 }
             }
@@ -280,12 +324,13 @@ struct gen_gemm_t : public gpu_gemm_t {
                     ? attr_zps.get_data_type(DNNL_ARG_A)
                     : data_type::s32;
             auto bo_type = data_type::s32;
+            bool int_acc = utils::one_of(eff_a_type(), s8, u8);
             auto co_type = with_bias() ? d->bias_type()
-                    : with_sum_ab()
-                    ? d->sum_ab_type
-                    : (utils::one_of(eff_a_type(), s8, u8) ? s32 : d->c_type());
+                    : with_sum_ab()    ? d->sum_ab_type
+                    : int_acc          ? s32
+                                       : d->c_type();
 
-            auto acc_type = utils::one_of(eff_a_type(), s8, u8)
+            auto acc_type = int_acc
                     ? s32
                     : (utils::one_of(f64, eff_a_type(), eff_b_type()) ? f64
                                                                       : f32);
@@ -309,6 +354,8 @@ struct gen_gemm_t : public gpu_gemm_t {
                 set_mode(mode, kernel_desc_t::mode_tf32);
             if (attr()->mayiconvert(f32, bf16))
                 set_mode(mode, kernel_desc_t::mode_bf16x1);
+            if (attr()->mayiconvert(f32, f16))
+                set_mode(mode, kernel_desc_t::mode_f16x1);
             if (attr()->deterministic_)
                 set_mode(mode, kernel_desc_t::mode_deterministic);
 
@@ -324,10 +371,11 @@ struct gen_gemm_t : public gpu_gemm_t {
             CHECK(kernel_desc_.select_kernel(arch_, stepping,
                     dev_info_->eu_count(), has_systolic, mode, batch_dims(),
                     eff_transa(), eff_transb(), eff_trans_bias(), swap_ab(),
-                    ao_dims_, bo_dims_, wei_scales_2d_, wei_q2d_group_k,
-                    with_c_zero_points(), with_bias(), eff_sum_ab(), alpha(),
-                    beta(), eff_a_type(), eff_b_type(), desc()->c_type(),
-                    ao_type, bo_type, wei_scales_type, co_type, acc_type,
+                    ao_dims_, bo_dims_, wei_scales_2d_, src_scales_2d_,
+                    wei_q2d_group_k, src_q2d_group_k, with_c_zero_points(),
+                    with_bias(), eff_sum_ab(), alpha(), beta(), eff_a_type(),
+                    eff_b_type(), desc()->c_type(), ao_type, bo_type,
+                    wei_scales_type, src_scales_type, co_type, acc_type,
                     eff_align_a(), eff_align_b(), align_c(), eff_m(), eff_n(),
                     d->k(), eff_lda(), eff_ldb(), d->ldc(), d->batch(),
                     std::move(gpu_post_ops)));
@@ -376,8 +424,6 @@ struct gen_gemm_t : public gpu_gemm_t {
             auto a_t = (utils::one_of(d->a_type(), s4, u4)) ? s8 : d->a_type();
             auto b_t = (utils::one_of(d->b_type(), s4, u4)) ? s8 : d->b_type();
             auto c_t = d->c_type();
-            auto a_t_sz = types::data_type_size(a_t);
-            auto b_t_sz = types::data_type_size(b_t);
 
             bool is_f16 = utils::everyone_is(f16, a_t, b_t, c_t);
             bool is_xe_hp_plus = arch_ >= arch_t::xe_hp;
@@ -410,16 +456,16 @@ struct gen_gemm_t : public gpu_gemm_t {
             auto lda = is_a_trans ? m : k;
             auto ldb = is_b_trans ? k : n;
 
-            auto is_aligned = [](dim_t ld, size_t sz, int byte) {
-                return ld * sz % byte == 0;
+            auto is_aligned = [](dim_t ld, data_type_t dt, int byte) {
+                return types::elements_to_bytes(dt, ld) % byte == 0;
             };
 
-            bool a_4B_aligned = is_aligned(lda, a_t_sz, 4);
-            bool b_4B_aligned = is_aligned(ldb, b_t_sz, 4);
+            bool a_4B_aligned = is_aligned(lda, a_t, 4);
+            bool b_4B_aligned = is_aligned(ldb, b_t, 4);
             bool ab_4B_aligned = a_4B_aligned && b_4B_aligned;
 
-            bool a_tn_4B_aligned = is_aligned(k, a_t_sz, 4);
-            bool b_tn_4B_aligned = is_aligned(k, b_t_sz, 4);
+            bool a_tn_4B_aligned = is_aligned(k, a_t, 4);
+            bool b_tn_4B_aligned = is_aligned(k, b_t, 4);
             bool ab_tn_4B_aligned = a_tn_4B_aligned && b_tn_4B_aligned;
 
             bool use_tn = (m <= 32 || n <= 32) && !ab_4B_aligned
@@ -505,6 +551,7 @@ struct gen_gemm_t : public gpu_gemm_t {
         }
 
         bool wei_scales_2d() const { return wei_scales_2d_; }
+        bool src_scales_2d() const { return src_scales_2d_; }
 
         bool swap_ab() const { return swap_ab_; }
 
@@ -534,27 +581,33 @@ struct gen_gemm_t : public gpu_gemm_t {
             return !swap_ab() ? desc()->b_type() : desc()->a_type();
         }
         int eff_align_a() const {
-            auto sz = types::data_type_size(eff_a_type());
-            auto align = utils::max_pow2_div(eff_lda() * sz);
+            auto dt = eff_a_type();
+            auto align = utils::max_pow2_div(
+                    types::elements_to_bytes(dt, eff_lda()));
             for (int b = 0; b < batch_dims(); b++)
-                align = nstl::min(
-                        align, utils::max_pow2_div(eff_stride_a(b) * sz));
+                align = nstl::min(align,
+                        utils::max_pow2_div(
+                                types::elements_to_bytes(dt, eff_stride_a(b))));
             return int(align);
         }
         int eff_align_b() const {
-            auto sz = types::data_type_size(eff_b_type());
-            auto align = utils::max_pow2_div(eff_ldb() * sz);
+            auto dt = eff_b_type();
+            auto align = utils::max_pow2_div(
+                    types::elements_to_bytes(dt, eff_ldb()));
             for (int b = 0; b < batch_dims(); b++)
-                align = nstl::min(
-                        align, utils::max_pow2_div(eff_stride_b(b) * sz));
+                align = nstl::min(align,
+                        utils::max_pow2_div(
+                                types::elements_to_bytes(dt, eff_stride_b(b))));
             return int(align);
         }
         int align_c() const {
-            auto sz = types::data_type_size(desc()->c_type());
-            auto align = utils::max_pow2_div(desc()->ldc() * sz);
+            auto dt = desc()->c_type();
+            auto align = utils::max_pow2_div(
+                    types::elements_to_bytes(dt, desc()->ldc()));
             for (int b = 0; b < batch_dims(); b++)
-                align = nstl::min(
-                        align, utils::max_pow2_div(desc()->stride_c(b) * sz));
+                align = nstl::min(align,
+                        utils::max_pow2_div(types::elements_to_bytes(
+                                dt, desc()->stride_c(b))));
             return int(align);
         }
 
@@ -582,7 +635,11 @@ struct gen_gemm_t : public gpu_gemm_t {
         int ao_dims_ = -1, bo_dims_ = -1;
         bool a_zp_ = false, b_zp_ = false;
         bool wei_decomp_ = false;
+        bool quant_enabled_ = false;
+        bool dy_quant_enabled_ = false;
         bool wei_scales_2d_ = false;
+        bool src_scales_2d_ = false;
+        bool src_po_sc_ = false;
         dim_t eff_lda_ = 0, eff_ldb_ = 0;
         bool eff_transa_ = false, eff_transb_ = false;
 
@@ -598,25 +655,21 @@ struct gen_gemm_t : public gpu_gemm_t {
         if (zero_pool_) release_zero_pool(zero_pool_);
     }
 
-    status_t init(engine_t *engine) override { return init_nocopy(engine); }
+    status_t init(impl::engine_t *engine) override {
+        return init_nocopy(engine);
+    }
 
-    status_t init_nocopy(engine_t *engine) {
+    status_t init_nocopy(impl::engine_t *engine) {
         using namespace data_type;
 
         auto kd = pd()->kernel_desc();
+
         CHECK(create_kernel(engine, nocopy_kernel_, "gemm_kernel", *kd));
 
         scalar_type_ = kd->scalar_type();
         const auto *info = nocopy_info();
 
-        if (get_verbose(verbose_t::debuginfo) >= 2) {
-            printf("onednn_verbose,info,gpu,%s\n", kd->entry().str().c_str());
-        }
-
         if (info->fusedBeta() || info->fusedPostOps()) {
-            auto *compute_engine
-                    = utils::downcast<compute::compute_engine_t *>(engine);
-
             int zg_cl = 0;
             if (info->fusedBeta()) zg_cl++;
             if (info->fusedPostOps()) zg_cl++;
@@ -624,9 +677,14 @@ struct gen_gemm_t : public gpu_gemm_t {
             zero_pool_bytes_ = pd()->max_k_sliced_groups() * 64 * zg_cl;
 
             auto zg_max = pd()->dev_info_->hw_threads(false);
-            auto zg_bytes_max = zg_max * 2 * 2 * 64;
+            zero_pool_chunk_size_ = zg_max * 2 * 2 * 64;
 
-            CHECK(lookup_zero_pool(compute_engine, zg_bytes_max, &zero_pool_));
+#ifndef DNNL_WITH_SYCL
+            auto *compute_engine
+                    = utils::downcast<compute::compute_engine_t *>(engine);
+            CHECK(lookup_zero_pool(compute_engine, nullptr,
+                    zero_pool_chunk_size_, &zero_pool_));
+#endif
 
             nocopy_kernel_.save_output_events();
         }
@@ -638,17 +696,18 @@ struct gen_gemm_t : public gpu_gemm_t {
 
 private:
     status_t launch_nocopy(const gemm_exec_ctx_t &ctx,
-            compute::compute_stream_t *s, const memory_storage_t &a,
-            const memory_storage_t &b, const memory_storage_t &c,
-            const memory_storage_t *ao, const memory_storage_t *bo,
-            const memory_storage_t *a_scales, const memory_storage_t *b_scales,
-            const memory_storage_t &co, const memory_storage_t *c_temp,
-            int po_count, const memory_storage_t **po_src, int64_t offset_a,
-            int64_t offset_b, int64_t offset_c, int32_t offset_aq,
-            int32_t offset_bq, int32_t offset_co, int32_t *offset_po_src,
-            int32_t lda, int32_t ldb, int32_t ldc, int32_t m, int32_t n,
-            int32_t k, int32_t k0, float alpha, float beta, int32_t cmask,
-            bool last_k_block, bool swapab, bool disable_hilbert) const;
+            compute::compute_stream_t *s, zero_pool_t *zero_pool,
+            const memory_storage_t &a, const memory_storage_t &b,
+            const memory_storage_t &c, const memory_storage_t *ao,
+            const memory_storage_t *bo, const memory_storage_t *a_scales,
+            const memory_storage_t *b_scales, const memory_storage_t &co,
+            const memory_storage_t *c_temp, int po_count,
+            const memory_storage_t **po_src, int64_t offset_a, int64_t offset_b,
+            int64_t offset_c, int32_t offset_aq, int32_t offset_bq,
+            int32_t offset_co, int32_t *offset_po_src, int32_t lda, int32_t ldb,
+            int32_t ldc, int32_t m, int32_t n, int32_t k, int32_t k0,
+            float alpha, float beta, int32_t cmask, bool last_k_block,
+            bool swapab, bool disable_hilbert) const;
 
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
     const CommonDriverInfo *nocopy_info() const {
@@ -659,6 +718,7 @@ private:
     compute::scalar_type_t scalar_type_;
     zero_pool_t *zero_pool_ = nullptr;
     size_t zero_pool_bytes_ = 0;
+    size_t zero_pool_chunk_size_ = 0;
 };
 
 } // namespace jit
