@@ -30,38 +30,6 @@ namespace jit {
 expr_t const_fold_non_recursive(const expr_t &expr);
 object_t const_fold(const object_t &obj);
 
-std::string to_string(type_kind_t kind) {
-#define CASE(_kind) \
-    case type_kind_t::_kind: return #_kind
-    switch (kind) {
-        CASE(undef);
-        CASE(u8);
-        CASE(s8);
-        CASE(u16);
-        CASE(s16);
-        CASE(u32);
-        CASE(s32);
-        CASE(u64);
-        CASE(s64);
-        CASE(bf8);
-        CASE(hf8);
-        CASE(bf16);
-        CASE(f16);
-        CASE(tf32);
-        CASE(f32);
-        CASE(f64);
-        CASE(byte);
-        CASE(dword);
-        CASE(qword);
-        CASE(oword);
-        CASE(hword);
-        case type_kind_t::_bool: return "bool";
-        default: ir_error_not_expected();
-    }
-#undef CASE
-    return {};
-}
-
 int type_t::size() const {
     if (is_ptr()) return sizeof(uint64_t);
 
@@ -161,6 +129,20 @@ bool is_cmp_op(op_kind_t op_kind) {
     }
 }
 
+bool is_commutative_op(op_kind_t op_kind) {
+    switch (op_kind) {
+        case op_kind_t::_add:
+        case op_kind_t::_mul:
+        case op_kind_t::_min:
+        case op_kind_t::_max:
+        case op_kind_t::_eq:
+        case op_kind_t::_ne:
+        case op_kind_t::_and:
+        case op_kind_t::_add3: return true;
+        default: return false;
+    }
+}
+
 op_kind_t negate_cmp_op(op_kind_t op_kind) {
     switch (op_kind) {
         case op_kind_t::_ge: return op_kind_t::_le;
@@ -378,7 +360,7 @@ DEFINE_BINARY_OPERATOR(&, op_kind_t::_and)
 #define DEFINE_BINARY_ASSIGN_OPERATOR(op) \
     expr_t &expr_t::operator op##=(const expr_t &rhs) { \
         auto tmp = (*this)op rhs; \
-        *this = tmp; \
+        *this = std::move(tmp); \
         return *this; \
     }
 
