@@ -227,7 +227,31 @@ enum class type_kind_t {
     hword
 };
 
-std::string to_string(type_kind_t kind);
+static auto type_kind_names = nstl::to_array({
+        make_enum_name(type_kind_t::undef, "undef"),
+        make_enum_name(type_kind_t::u8, "u8"),
+        make_enum_name(type_kind_t::s8, "s8"),
+        make_enum_name(type_kind_t::u16, "u16"),
+        make_enum_name(type_kind_t::s16, "s16"),
+        make_enum_name(type_kind_t::u32, "u32"),
+        make_enum_name(type_kind_t::s32, "s32"),
+        make_enum_name(type_kind_t::u64, "u64"),
+        make_enum_name(type_kind_t::s64, "s64"),
+        make_enum_name(type_kind_t::bf8, "bf8"),
+        make_enum_name(type_kind_t::hf8, "hf8"),
+        make_enum_name(type_kind_t::bf16, "bf16"),
+        make_enum_name(type_kind_t::f16, "f16"),
+        make_enum_name(type_kind_t::tf32, "tf32"),
+        make_enum_name(type_kind_t::f32, "f32"),
+        make_enum_name(type_kind_t::f64, "f64"),
+        make_enum_name(type_kind_t::byte, "byte"),
+        make_enum_name(type_kind_t::dword, "dword"),
+        make_enum_name(type_kind_t::qword, "qword"),
+        make_enum_name(type_kind_t::oword, "oword"),
+        make_enum_name(type_kind_t::hword, "hword"),
+        make_enum_name(type_kind_t::_bool, "bool"),
+});
+GPU_DEFINE_PARSE_ENUM(type_kind_t, type_kind_names)
 
 class type_t {
 public:
@@ -426,16 +450,12 @@ public:
         return ir_utils::get_hash(kind(), elems(), is_ptr());
     }
 
-    void serialize(std::ostream &out) const {
-        ir_utils::serialize(kind_, out);
-        ir_utils::serialize(elems_, out);
-        ir_utils::serialize(is_ptr_, out);
-    }
-
-    void deserialize(std::istream &in) {
-        ir_utils::deserialize(kind_, in);
-        ir_utils::deserialize(elems_, in);
-        ir_utils::deserialize(is_ptr_, in);
+    static void init_parse_iface(parse_iface_t<type_t> *iface) {
+        iface->add<type_kind_t, &type_t::kind_>();
+        iface->set_pre_stringify_func([](const type_t &type) {
+            ir_assert(!type.is_ptr() && type.is_scalar())
+                    << "Cannot stringify pointer/non-scalar type.";
+        });
     }
 
     bool is_undef() const { return kind() == type_kind_t::undef; }
@@ -450,6 +470,10 @@ public:
                 type_kind_t::f32, type_kind_t::f64);
     }
 
+    bool is_fp8() const {
+        return utils::one_of(kind(), type_kind_t::bf8, type_kind_t::hf8);
+    }
+
     bool is_bf8() const { return kind() == type_kind_t::bf8; }
     bool is_hf8() const { return kind() == type_kind_t::hf8; }
     bool is_bf16() const { return kind() == type_kind_t::bf16; }
@@ -593,8 +617,7 @@ HANDLE_TRAVERSE_TARGETS()
 // the reference counter stored inside the object.
 class object_impl_t {
 public:
-    object_impl_t(type_info_t type_info)
-        : ref_count_(), type_info_(type_info) {};
+    object_impl_t(type_info_t type_info) : type_info_(type_info) {};
 
     object_impl_t(const object_impl_t &) = delete;
 
@@ -689,6 +712,7 @@ public:
 #endif
 
     object_t &operator=(const object_t &other) {
+        if (&other == this) return *this;
         auto *other_impl = other.impl();
         increment(other_impl);
         decrement_and_maybe_destroy(impl_);
@@ -1016,6 +1040,8 @@ inline std::ostream &operator<<(std::ostream &out, op_kind_t kind) {
 
 bool is_cmp_op(op_kind_t op_kind);
 
+bool is_commutative_op(op_kind_t op_kind);
+
 op_kind_t negate_cmp_op(op_kind_t op_kind);
 
 type_t unary_op_type(op_kind_t op_kind, const expr_t &a);
@@ -1852,7 +1878,7 @@ public:
 
     stmt_t() = default;
     stmt_t(const object_t &obj) : object_t(obj) {}
-    stmt_t(object_t &&obj) : object_t(obj) {}
+    stmt_t(object_t &&obj) : object_t(std::move(obj)) {}
     stmt_t &operator=(const object_t &obj) {
         object_t::operator=(obj);
         return *this;
@@ -2074,18 +2100,25 @@ public:
     // default stride means unit stride (in terms of value.type().scalar()
     // elements).
     static stmt_t make(const expr_t &buf, const expr_t &off,
-            const expr_t &value, int stride = default_stride,
+            const expr_t &_value, int stride = default_stride,
             const expr_t &_mask = expr_t(), bool fill_mask0 = false) {
         auto mask = _mask;
+        auto value = _value;
         if (!mask.is_empty()) {
             if (all_of(mask, expr_t(true))) {
                 mask = expr_t();
             } else if (all_of(mask, expr_t(false))) {
-                // No need to store anything with a false mask.
-                return stmt_t();
+                // No need to store anything with a false mask,
+                // unless explicitly asked to zero-fill the rest.
+                if (!fill_mask0) return stmt_t();
+                auto type = value.type();
+                value = shuffle_t::make_broadcast(
+                        cast_t::make(type.scalar(), 0), type.elems());
+                mask = expr_t();
             }
         }
-        return stmt_t(new store_t(buf, off, value, stride, mask, fill_mask0));
+        return stmt_t(new store_t(
+                buf, off, value, stride, mask, fill_mask0 && !mask.is_empty()));
     }
 
     bool is_equal(const object_impl_t &obj) const override {
@@ -2316,6 +2349,10 @@ public:
         return index_ == other.index_;
     }
 
+    bool operator!=(const stmt_label_t &other) const {
+        return !operator==(other);
+    }
+
     size_t get_hash() const { return ir_utils::get_hash(kind_, index_); }
 
     std::string str() const {
@@ -2511,6 +2548,16 @@ public:
 
     func_impl_t(type_info_t type_info) : object_impl_t(type_info) {}
 
+    size_t get_hash() const override {
+        ir_error_not_expected() << "get_hash() is not implemented.";
+        return 0;
+    }
+
+    bool is_equal(const object_impl_t &obj) const override {
+        ir_error_not_expected() << "is_equal() is not implemented.";
+        return false;
+    }
+
     stmt_t call(const std::vector<expr_t> &args,
             const func_call_attr_t &attr = {}) const;
 
@@ -2566,9 +2613,7 @@ public:
                 && attr.is_equal(other.attr);
     }
 
-    size_t get_hash() const override {
-        return ir_utils::get_hash(func, args, attr);
-    }
+    size_t get_hash() const override { return ir_utils::get_hash(args, attr); }
 
     IR_DECLARE_TRAVERSERS()
 
@@ -2619,8 +2664,6 @@ public:
         return name == other.name;
     }
 
-    size_t get_hash() const override { return ir_utils::get_hash(name); }
-
     std::string str() const override { return name; }
 
     std::string name;
