@@ -646,6 +646,7 @@ struct mem_usage_guard_t {
     }
 
     mem_usage_guard_t &operator=(mem_usage_guard_t &&other) {
+        if (&other == this) return *this;
         usage = other.usage;
         peak_usage = other.peak_usage;
         size = other.size;
@@ -696,7 +697,8 @@ public:
     bool implies(const relation_t &other) const;
 
     // Applies linear transformation to left and right hand sides of the relation.
-    relation_t transform(const linear_transform_t &t, const expr_t &new_var);
+    relation_t transform(
+            const linear_transform_t &t, const expr_t &new_var) const;
 
     std::string str() const {
         std::ostringstream oss;
@@ -841,6 +843,33 @@ public:
 
     int max_proven_gcd(const expr_t &var) const;
 
+    std::string str() const {
+        std::ostringstream oss;
+        oss << "relations:" << (relations_.empty() ? " (empty)\n" : "\n");
+        for (auto &r : relations_) {
+            oss << "\t" << r.first << ":";
+            bool first = true;
+            for (auto &s : r.second) {
+                oss << (first ? " " : ", ") << s;
+                first = false;
+            }
+            oss << "\n";
+        }
+
+        oss << "modulus_info:"
+            << (modulus_infos_.empty() ? " (empty)\n" : "\n");
+        for (auto &m : modulus_infos_) {
+            oss << "\t" << m.first << ":";
+            bool first = true;
+            for (auto &s : m.second) {
+                oss << (first ? " " : ", ") << s;
+                first = false;
+            }
+            oss << "\n";
+        }
+        return oss.str();
+    }
+
 private:
     bool can_prove_modulus(const expr_t &e) const {
         modulus_info_t unknown(e);
