@@ -34,60 +34,101 @@ namespace jit {
 namespace v2 {
 namespace conv {
 
-load_desc_t str_to_load_desc(const std::string &s) {
+void load_desc_t::parse(std::istream &in) {
+    operator=(load_desc_t());
+    a = send_kind_t::undef;
+    b = send_kind_t::undef;
+    std::string s;
+    in >> s;
+    if (s == "x") return;
     auto parts = gpu_utils::split(s, ",");
-    load_desc_t ret;
     for (auto &p : parts) {
         auto p_parts = gpu_utils::split(p, ":");
         ir_assert(p_parts.size() == 2);
         auto tensor = p_parts[0];
         auto kind = p_parts[1];
         if (tensor == "a") {
-            ret.a = str_to_send_kind(kind);
+            a = to_enum<send_kind_t>(kind);
         } else if (tensor == "b") {
-            ret.b = str_to_send_kind(kind);
+            b = to_enum<send_kind_t>(kind);
         } else {
             ir_error_not_expected() << p;
         }
     }
-    return ret;
 }
 
-store_desc_t str_to_store_desc(const std::string &s) {
+void store_desc_t::parse(std::istream &in) {
+    operator=(store_desc_t());
+    std::string s;
+    in >> s;
+    if (s == "x") return;
     auto parts = gpu_utils::split(s, ",");
-    store_desc_t ret;
     for (auto &p : parts) {
         auto p_parts = gpu_utils::split(p, ":");
         ir_assert(p_parts.size() == 2);
         auto tensor = p_parts[0];
         auto kind = p_parts[1];
         if (tensor == "c") {
-            ret.c = str_to_send_kind(kind);
+            c = to_enum<send_kind_t>(kind);
         } else {
             ir_error_not_expected() << p;
         }
     }
-    return ret;
 }
 
-prefetch_desc_t str_to_prefetch_desc(const std::string &s) {
+std::string align_desc_t::align_t::str() const {
+    std::string s = (value == 0 ? "*" : std::to_string(value));
+    if (in_bytes) s += "b";
+    return s;
+}
+
+void align_desc_t::align_t::parse(const std::string &_s) {
+    auto s = _s;
+    in_bytes = (!s.empty() && s.back() == 'b');
+    if (in_bytes) s = s.substr(0, s.length() - 1);
+    value = (s == "*") ? 0 : std::stoi(s);
+}
+
+std::string align_desc_t::str() const {
+    if (is_default()) return "x";
+    std::vector<std::string> parts;
+    parts.emplace_back(src.str());
+    parts.emplace_back(wei.str());
+    parts.emplace_back(dst.str());
+    return gpu_utils::join(":", parts);
+}
+
+void align_desc_t::parse(std::istream &in) {
+    operator=(align_desc_t());
+    auto s = jit::parse<std::string>(in);
+    if (s == "x") return;
+    auto parts = gpu_utils::split(s, ":");
+    if (parts.size() == 1) {
+        parts.push_back(parts[0]);
+        parts.push_back(parts[0]);
+    }
+    ir_assert(parts.size() == 3);
+    src.parse(parts[0]);
+    wei.parse(parts[1]);
+    dst.parse(parts[2]);
+}
+
+void prefetch_desc_t::parse(std::istream &in) {
+    operator=(prefetch_desc_t());
+    std::string s;
+    in >> s;
     auto parts = gpu_utils::split(s, ".");
     ir_assert(utils::one_of((int)parts.size(), 1, 2));
     ir_assert(parts[0].size() >= 2);
-    int dist = std::stoi(parts[0].substr(1));
+    dist = std::stoi(parts[0].substr(1));
     ir_assert(dist >= 0);
-    bool a = (dist > 0);
-    bool b = (dist > 0);
+    a = (dist > 0);
+    b = (dist > 0);
     if (parts.size() == 2 && dist > 0) {
         ir_assert(utils::one_of(parts[1], "a", "b", "ab"));
         a = (parts[1].find("a") != std::string::npos);
         b = (parts[1].find("b") != std::string::npos);
     }
-    prefetch_desc_t ret;
-    ret.dist = dist;
-    ret.a = a;
-    ret.b = b;
-    return ret;
 }
 
 layout_desc_t make_conv_layout_desc(
@@ -121,6 +162,7 @@ layout_desc_t make_conv_algo_layout_desc(
         prop_kind_t prop, tensor_kind_t tensor_kind) {
     auto desc = make_conv_layout_desc(tensor_kind, /*src_dst_with_group=*/true);
     switch (tensor_kind) {
+        case tensor_kind_t::bia:
         case tensor_kind_t::wei: return desc;
         case tensor_kind_t::src:
             if (prop == prop_kind::backward_data) return desc;
@@ -249,6 +291,27 @@ layout_tag_t make_conv_layout_tag(
     raw_tag = normalize_conv_tag(tensor_kind, conv_ndims, raw_tag);
     return layout_tag_t(desc, type, raw_tag);
 }
+prb_tile_t min_dims_tile(const problem_t &prb) {
+    prb_tile_t xd;
+    xd[prb_dims::id] = xd[prb_dims::od] = xd[prb_dims::kd] = 1;
+    xd[prb_dims::dd] = xd[prb_dims::pd] = 0;
+    xd[prb_dims::sd] = 1;
+    prb_tile_t xhd = xd;
+    xhd[prb_dims::ih] = xhd[prb_dims::oh] = xhd[prb_dims::kh] = 1;
+    xhd[prb_dims::dh] = xhd[prb_dims::ph] = 0;
+    xhd[prb_dims::sh] = 1;
+    for (auto *t : {&xhd, &xd}) {
+        bool ok = true;
+        for (auto &d : *t) {
+            if (prb.shape().at(d) != (*t).at(d)) {
+                ok = false;
+                break;
+            }
+        }
+        if (ok) return *t;
+    }
+    return prb_tile_t();
+}
 
 int estimate_grf_usage_bytes(const kernel_desc_t &desc) {
     int a_type_size = desc.a_type().size();
@@ -262,6 +325,15 @@ int estimate_grf_usage_bytes(const kernel_desc_t &desc) {
     int a_elems = b_iter * m_iter * k_iter;
     int b_elems = b_iter * k_iter * n_iter;
     int c_elems = m_iter * n_iter;
+    auto iter_outer_dim
+            = (desc.iter_outer_tile.is_empty() ? prb_dims::undef
+                                               : *desc.iter_outer_tile.begin());
+    auto bmnk = to_gemm(iter_outer_dim, desc.prop);
+    if (bmnk == prb_dims::m) {
+        a_elems = utils::div_up(a_elems, desc.iter_outer_tile.elems());
+    } else if (bmnk == prb_dims::n) {
+        b_elems = utils::div_up(b_elems, desc.iter_outer_tile.elems());
+    }
     int a_size = a_elems * a_type_size;
     int a_reorder_size = 0;
     int b_size = b_elems * b_type_size;
@@ -302,8 +374,8 @@ bool kernel_desc_t::is_supported() const {
 void kernel_desc_t::set(const std::string &s) {
     operator=(kernel_desc_t());
     if (s.empty()) return;
-    auto iface = cli_iface();
-    iface.parse(s, this);
+    auto iface = parse_iface();
+    iface.parse(s, *this);
     set_defaults();
 }
 
@@ -332,40 +404,124 @@ void kernel_desc_t::set_defaults() {
             default: ir_error_not_expected(); break;
         }
     }
+    if (is_dw) {
+        reqs.set(prb_dims::ic, 1);
+        reqs.set(prb_dims::oc, 1);
+    }
+    if (prop == prop_kind::backward_weights && with_bias) {
+        bia_tag = make_conv_layout_tag(tensor_kind_t::bia, "a");
+        bia_tag = layout_tag_t(
+                bia_tag.desc(), dst_tag.type(), bia_tag.raw_tag());
+    }
 }
 
-void kernel_desc_t::finalize(const plan_t &plan) {
+void kernel_desc_t::finalize(const prb_reqs_t &final_reqs) {
     is_finalized = true;
-    reqs = plan.reqs();
+    reqs.add(final_reqs);
 }
 
 std::string kernel_desc_t::cmd_str() const {
-    return cli_iface().cmd_str(this);
+    return parse_iface().cmd_str(*this);
 }
 
 std::string kernel_desc_t::str() const {
     std::ostringstream oss;
-    oss << "Propagation:        " << ir_utils::to_string(prop) << std::endl;
-    oss << "Depthwise:          " << ir_utils::to_string(is_dw) << std::endl;
-    oss << "Source tag:         " << src_tag << std::endl;
-    oss << "Weights tag:        " << wei_tag << std::endl;
-    oss << "Destination tag:    " << dst_tag << std::endl;
-    oss << "Specialization:     " << spec_reqs << std::endl;
-    oss << "HW:                 " << ir_utils::to_lower(hw.str()) << std::endl;
-    oss << "FMA kind:           " << to_string(fma) << std::endl;
-    oss << "SIMD:               " << simd << std::endl;
-    oss << "Registers:          " << regs << std::endl;
-    oss << "Iteration tile:     " << iter_tile << std::endl;
-    oss << "Thread group tile:  " << thread_group_tile << std::endl;
-    oss << "Loop desc:          " << loop_desc << std::endl;
-    oss << "Load:               " << load.str() << std::endl;
-    oss << "Prefetch:           " << prefetch.str() << std::endl;
-    oss << "Store:              " << store.str() << std::endl;
+    oss << "Propagation:            " << jit::to_string(prop) << std::endl;
+    oss << "Depthwise:              " << ir_utils::to_string(is_dw)
+        << std::endl;
+    oss << "With bias:              " << ir_utils::to_string(with_bias)
+        << std::endl;
+    oss << "Source tag:             " << src_tag << std::endl;
+    oss << "Weights tag:            " << wei_tag << std::endl;
+    oss << "Destination tag:        " << dst_tag << std::endl;
+    oss << "HW:                     " << jit::to_string(hw.to_ngen())
+        << std::endl;
+    oss << "FMA kind:               " << to_string(fma) << std::endl;
+    oss << "SIMD:                   " << simd << std::endl;
+    oss << "Registers:              " << regs << std::endl;
+    oss << "Iteration tile:         " << iter_tile << std::endl;
+    oss << "Iteration outer tile:   " << iter_outer_tile << std::endl;
+    oss << "Thread group tile:      " << thread_group_tile << std::endl;
+    oss << "Loop desc:              " << loop_desc << std::endl;
+    oss << "Load:                   " << load.str() << std::endl;
+    oss << "Store:                  " << store.str() << std::endl;
+    oss << "Use block 2D access:    " << ir_utils::to_string(use_2d_access)
+        << std::endl;
+    oss << "Align:                  " << align.str() << std::endl;
+    oss << "Prefetch:               " << prefetch.str() << std::endl;
     if (reqs) oss << ir_utils::add_tag("Reqs", reqs.str()) << std::endl;
-    oss << "Command:            " << cmd_str();
+    oss << "Command:                " << cmd_str();
     return ir_utils::add_tag("Desc", oss.str());
 }
 
+void kernel_desc_t::init_parse_iface(parse_iface_t<kernel_desc_t> *iface) {
+    iface->set_relaxed(true);
+#define PACK(member) decltype(kernel_desc_t::member), &kernel_desc_t::member
+    iface->add<PACK(prop)>("prop", "Propagation kind (fwd, bwd_d or bwd_w).",
+            /*required=*/true);
+    iface->add<PACK(is_dw)>(
+            "dw", "Whether the problem is a depthwise convolution (0 or 1).");
+    iface->add<PACK(with_bias)>(
+            "with_bias", "Whether the problem has bias (0 or 1).");
+    iface->add<PACK(src_tag)>("src",
+            "Source layout tag. Examples: axb:f32, aBx16b:f16).",
+            /*required=*/true);
+    iface->add<PACK(wei_tag)>("wei", "Weights layout tag (e.g. axcb:f32).",
+            /*required=*/true);
+    iface->add<PACK(dst_tag)>("dst", "Destination layout tag (e.g. axb:f32).",
+            /*required=*/true);
+    iface->add<PACK(hw_desc)>("hw", "Hardware (xehpc).", /*required=*/true);
+    iface->add<PACK(fma)>("fma", "FMA kind (e.g. mad).", /*required=*/true);
+    iface->add<PACK(simd)>("simd", "SIMD size (16 or 32).", /*required=*/true);
+    iface->add<PACK(regs)>(
+            "regs", "Number of registers (128 or 256).", /*required=*/true);
+    iface->add<PACK(iter_tile)>("iter", "Iteration tile (e.g. mb32ic16oc16).",
+            /*required=*/true);
+    iface->add<PACK(iter_outer_tile)>("iter_outer",
+            "Outer iteration tile (e.g. mb2).",
+            /*required=*/false);
+    iface->add<PACK(thread_group_tile)>(
+            "tg", "Threadgroup tile (e.g. ow4oc4).", /*required=*/true);
+    iface->add<PACK(loop_desc)>("loop_desc",
+            "Loop description, variables ordered from innermost to outermost "
+            "(e.g. kw,kh,kd,ic).");
+    iface->add<PACK(load)>("load",
+            "Load type (block, scattered [default], 2d) for A and B, e.g. "
+            "a:2d,b:block.");
+    iface->add<PACK(store)>("store",
+            "Store type (block, scattered [default], 2d) for C,  e.g. c:2d.");
+    iface->add<PACK(use_2d_access)>(
+            "2d", "Whether to use block 2D messages for access.");
+    iface->add<PACK(align)>("align",
+            "Alignments in bytes/elements for the innermost dimension in "
+            "source, weights and destination. Examples: 8b:8b:8b (in bytes), "
+            "2:2:2 (in elements), *:*:* (for optimal values determined during "
+            "kernel plan generation).");
+    iface->add<PACK(prefetch)>("prefetch",
+            "Prefetch description specifying distance and whether A/B are "
+            "prefetched. Examples: x3 (distance is 3, both A/B are "
+            "prefetched), x2.a (distance is 2, only A is prefetched), x0 (no "
+            "prefetch, default).");
+    iface->add<PACK(spec_strategy)>("spec_strategy",
+            "Specialization strategy for problem dimensions (e.g. min_dims to "
+            "eliminate unused spatial dimensions).");
+    iface->add<PACK(reqs)>("reqs",
+            "Dimension requirements, colon-separated (e.g. kd=1:mb>=16).");
+#undef PACK
+
+    iface->set_pre_stringify_func(
+            [](const kernel_desc_t &desc) { ir_assert(desc.is_finalized); });
+    iface->set_post_parse_func([](kernel_desc_t &desc) {
+        desc.src_tag
+                = make_conv_layout_tag(tensor_kind_t::src, desc.src_tag.str());
+        desc.wei_tag
+                = make_conv_layout_tag(tensor_kind_t::wei, desc.wei_tag.str());
+        desc.dst_tag
+                = make_conv_layout_tag(tensor_kind_t::dst, desc.dst_tag.str());
+        desc.is_finalized = true;
+    });
+}
+
 void init_kernel_info_div_magic(
         kernel_info_t &kernel_info, const kernel_desc_t &desc) {
     auto tg_grid = create_thread_group_grid(desc);
@@ -387,8 +543,18 @@ void init_dispatch_kernel_info_div_magic(
     }
 }
 
+compute::range_t kernel_desc_t::local_range() const {
+    auto thr_grid = create_thread_grid(*this);
+    compute::range_t lws = compute::range_t::empty();
+    for (size_t i = 0; i < compute::range_t::max_ndims; i++) {
+        size_t tg_dim = thr_grid.size(i, thread_group_tile);
+        lws[i] = tg_dim * (i == 0 ? gpu_utils::into<size_t>(simd) : 1);
+    }
+    return lws;
+}
+
 status_t kernel_desc_t::init_kernel_info(kernel_info_t &kernel_info) const {
-    auto tensor_config = get_tensor_config(prop);
+    auto tensor_config = get_tensor_config(prop, with_bias);
     for (auto &t : tensor_config.tensors()) {
         auto buf = make_buffer(t.name);
         kernel_info.register_user_arg(buf, t.arg_key, t.is_input);
@@ -403,7 +569,7 @@ status_t kernel_desc_t::init_kernel_info(kernel_info_t &kernel_info) const {
 }
 
 status_t kernel_desc_t::create_kernel(compute::kernel_t &kernel,
-        gpu_primitive_t *primitive, engine_t *engine) const {
+        gpu_primitive_t *primitive, impl::engine_t *engine) const {
     return primitive->create_kernel(
             engine, kernel, kernel_name().c_str(), *this);
 }
@@ -417,7 +583,7 @@ status_t kernel_desc_t::create_generator(
 
 serialized_t kernel_desc_t::serialize() const {
     std::ostringstream oss;
-    serialize(oss);
+    jit::stringify(oss, *this);
     auto str = oss.str();
     return serialized_t::from_data(
             std::vector<uint8_t>(str.begin(), str.end()));
@@ -427,91 +593,16 @@ kernel_desc_t kernel_desc_t::deserialize(const serialized_t &s) {
     auto &data = s.get_data();
     std::string str(data.begin(), data.end());
     std::istringstream iss(str);
-    kernel_desc_t desc;
-    desc.deserialize(iss);
+    auto desc = jit::parse<kernel_desc_t>(iss);
     return desc;
 }
 
-ir_utils::cli_iface_t<kernel_desc_t> kernel_desc_t::cli_iface() {
-#define MAKE_SETTER(lhs, rhs) \
-    *static_cast<void (*)(kernel_desc_t *, const std::string &)>( \
-            [](kernel_desc_t *desc, const std::string &value) { \
-                desc->lhs = rhs; \
-            })
-#define MAKE_GETTER(value) \
-    *static_cast<std::string (*)(const kernel_desc_t *)>( \
-            [](const kernel_desc_t *desc) { return value; })
-    ir_utils::cli_iface_t<kernel_desc_t> iface;
-    iface.add_arg("--prop", "Propagation kind (fwd, bwd_d or bwd_w).",
-            MAKE_GETTER(ir_utils::to_string(desc->prop)),
-            MAKE_SETTER(prop, ir_utils::str_to_prop_kind(value)));
-    iface.add_arg("--dw",
-            "Whether the problem is a depthwise convolution (0 or 1).",
-            MAKE_GETTER(std::string(desc->is_dw ? "1" : "0")),
-            MAKE_SETTER(is_dw, ir_utils::str_to_bool(value)));
-    iface.add_arg("--src", "Source layout tag. Examples: axb:f32, aBx16b:f16).",
-            MAKE_GETTER(desc->src_tag.str()),
-            MAKE_SETTER(
-                    src_tag, make_conv_layout_tag(tensor_kind_t::src, value)));
-    iface.add_arg("--wei", "Weights layout tag (e.g. axcb:f32).",
-            MAKE_GETTER(desc->wei_tag.str()),
-            MAKE_SETTER(
-                    wei_tag, make_conv_layout_tag(tensor_kind_t::wei, value)));
-    iface.add_arg("--dst", "Destination layout tag (e.g. axb:f32).",
-            MAKE_GETTER(desc->dst_tag.str()),
-            MAKE_SETTER(
-                    dst_tag, make_conv_layout_tag(tensor_kind_t::dst, value)));
-    iface.add_arg("--spec-reqs",
-            "Specialization requirements for problem dimensions (e.g. "
-            "kd1kw1kh1 for convolution without filter).",
-            MAKE_GETTER(desc->spec_reqs.str()),
-            MAKE_SETTER(spec_reqs, str_to_spec_reqs(value)));
-    iface.add_arg("--hw", "Hardware (xehpc).",
-            MAKE_GETTER(ir_utils::to_lower(jit::to_string(desc->hw.to_ngen()))),
-            MAKE_SETTER(hw, str_to_hw(value)));
-    iface.add_arg("--fma", "FMA kind (mad).", MAKE_GETTER(to_string(desc->fma)),
-            MAKE_SETTER(fma, str_to_fma_kind(value)));
-    iface.add_arg("--simd", "SIMD size (16 or 32).",
-            MAKE_GETTER(std::to_string(desc->simd)),
-            MAKE_SETTER(simd, std::stoi(value)));
-    iface.add_arg("--regs", "Number of registers (128 or 256).",
-            MAKE_GETTER(std::to_string(desc->regs)),
-            MAKE_SETTER(regs, std::stoi(value)));
-    iface.add_arg("--iter", "Iteration tile (e.g. mb32ic16oc16).",
-            MAKE_GETTER(desc->iter_tile.str()),
-            MAKE_SETTER(iter_tile, str_to_prb_tile(value)));
-    iface.add_arg("--tg", "Threadgroup tile (e.g. ow4oc4).",
-            MAKE_GETTER(desc->thread_group_tile.str()),
-            MAKE_SETTER(thread_group_tile, str_to_prb_tile(value)));
-    iface.add_arg("--loop-desc",
-            "Loop description, variables ordered from innermost to outermost "
-            "(e.g. kw,kh,kd,ic).",
-            MAKE_GETTER(desc->loop_desc.str()),
-            MAKE_SETTER(loop_desc, str_to_loop_desc(value)));
-    iface.add_arg("--load",
-            "Load type (block, scattered [default], 2d) for A and B, e.g. "
-            "a:2d,b:block.",
-            MAKE_GETTER(desc->load.str()),
-            MAKE_SETTER(load, str_to_load_desc(value)));
-    iface.add_arg("--store",
-            "Store type (block, scattered [default], 2d) for C,  e.g. c:2d.",
-            MAKE_GETTER(desc->store.str()),
-            MAKE_SETTER(store, str_to_store_desc(value)));
-    iface.add_arg("--prefetch",
-            "Prefetch description specifying distance and whether A/B are "
-            "prefetched. Examples: x3 (distance is 3, both A/B are "
-            "prefetched), x2.a (distance is 2, only A is prefetched), x0 (no "
-            "prefetch, default).",
-            MAKE_GETTER(desc->prefetch.str()),
-            MAKE_SETTER(prefetch, str_to_prefetch_desc(value)));
-    return iface;
-#undef MAKE_SETTER
-#undef MAKE_GETTER
+const parse_iface_t<kernel_desc_t> &kernel_desc_t::parse_iface() {
+    return parse_iface_helper_t<kernel_desc_t>::get();
 }
 
 void kernel_desc_t::show_help() {
-    kernel_desc_t desc;
-    desc.set("--help");
+    parse_iface().print_help();
 }
 
 grid_t create_thread_group_grid(const kernel_desc_t &desc) {
