@@ -18,6 +18,7 @@
 #define GPU_INTEL_JIT_V2_IR_SEND_HPP
 
 #include "gpu/intel/jit/ir/block_2d_utils.hpp"
+#include "gpu/intel/jit/ir/fma.hpp"
 #include "gpu/intel/jit/v2/ir/plan_utils.hpp"
 #include "gpu/intel/jit/v2/ir/reqs.hpp"
 #include "gpu/intel/jit/v2/ir/tensor.hpp"
@@ -42,20 +43,29 @@ enum class send_op_t {
     store,
 };
 
-inline std::string to_string(send_op_t kind) {
-    switch (kind) {
-#define CASE(name) \
-    case send_op_t::name: return #name
-        CASE(undef);
-        CASE(atomic_fadd);
-        CASE(load);
-        CASE(prefetch);
-        CASE(store);
-#undef CASE
-        default: ir_error_not_expected();
-    }
-    return {};
-}
+static auto send_op_names = nstl::to_array({
+        make_enum_name(send_op_t::undef, "undef"),
+        make_enum_name(send_op_t::atomic_fadd, "atomic_fadd"),
+        make_enum_name(send_op_t::load, "load"),
+        make_enum_name(send_op_t::prefetch, "prefetch"),
+        make_enum_name(send_op_t::store, "store"),
+});
+
+GPU_DEFINE_PARSE_ENUM(send_op_t, send_op_names)
+
+enum class send_address_t {
+    undef,
+    a64,
+    slm,
+};
+
+static auto send_address_names = nstl::to_array({
+        make_enum_name(send_address_t::undef, "undef"),
+        make_enum_name(send_address_t::a64, "a64"),
+        make_enum_name(send_address_t::slm, "slm"),
+});
+
+GPU_DEFINE_PARSE_ENUM(send_address_t, send_address_names)
 
 enum class send_kind_t {
     undef,
@@ -65,34 +75,15 @@ enum class send_kind_t {
     compressed_prefetch,
 };
 
-inline std::string to_string(send_kind_t kind) {
-    switch (kind) {
-#define CASE(name) \
-    case send_kind_t::name: return #name
-        CASE(undef);
-        CASE(block);
-        CASE(scattered);
-        CASE(compressed_prefetch);
-        case send_kind_t::_2d: return "2d";
-        default: ir_error_not_expected(); return "undef";
-#undef CASE
-    }
-}
+static auto send_kind_names = nstl::to_array({
+        make_enum_name(send_kind_t::undef, "undef"),
+        make_enum_name(send_kind_t::_2d, "2d"),
+        make_enum_name(send_kind_t::block, "block"),
+        make_enum_name(send_kind_t::scattered, "scattered"),
+        make_enum_name(send_kind_t::compressed_prefetch, "compressed_prefetch"),
+});
 
-inline send_kind_t str_to_send_kind(const std::string &s) {
-    if (s == "2d") return send_kind_t::_2d;
-#define CASE(name) \
-    do { \
-        if (s == to_string(send_kind_t::name)) return send_kind_t::name; \
-    } while (false)
-    CASE(undef);
-    CASE(block);
-    CASE(scattered);
-    CASE(compressed_prefetch);
-#undef CASE
-    ir_error_not_expected();
-    return send_kind_t::undef;
-}
+GPU_DEFINE_PARSE_ENUM(send_kind_t, send_kind_names)
 
 struct addr_t {
     expr_t base;
@@ -106,7 +97,7 @@ struct addr_t {
         for (int i = 1; i < slots; i++) {
             it.next(elems_per_slot);
             slot_incs[i] = simplify_rewrite(
-                    layout.offset_in_bytes(it.block_offset()));
+                    layout.shift_in_bytes(it.block_offset()));
         }
     }
 
@@ -327,11 +318,13 @@ struct send_2d_hint_t {
 
 struct send_params_t {
     hw_t hw;
+    send_address_t address = send_address_t::undef;
     send_kind_t kind = send_kind_t::undef;
     send_op_t op = send_op_t::undef;
     send_2d_hint_t hint_2d;
     // For register payload.
     int max_entry_reg_size = 0;
+    const prb_reqs_t *external_reqs = nullptr;
     std::vector<prb_dim_t> skip_mask;
 
     void init_max_entry_reg_size() {
@@ -351,6 +344,7 @@ struct send_params_t {
         std::ostringstream oss;
         oss << "send_params:" << std::endl;
         oss << "  hw:                 " << hw << std::endl;
+        oss << "  address:            " << to_string(address) << std::endl;
         oss << "  kind:               " << to_string(kind) << std::endl;
         if (hint_2d) oss << "  hint_2d:            " << hint_2d << std::endl;
         oss << "  max_entry_reg_size: " << max_entry_reg_size;
@@ -362,19 +356,20 @@ struct send_params_t {
 
 struct send_1d_desc_t {
     hw_t hw;
+    send_address_t address = send_address_t::undef;
     send_op_t op = send_op_t::undef;
     int type_size = 0;
     int slots = 0;
 
     explicit operator bool() const { return op != send_op_t::undef; }
 
-    bool base_alignment_ok(const expr_t &off, const prover_t &prover) {
+    bool base_alignment_ok(const expr_t &off, const prover_t &prover) const {
         int align = (type_size >= 16 ? 8 : 1);
         if (!prover.require(off % align == 0)) return false;
         return true;
     }
 
-    bool base_alignment_ok(const addr_t &addr, const prover_t &prover) {
+    bool base_alignment_ok(const addr_t &addr, const prover_t &prover) const {
         if (!base_alignment_ok(addr.base, prover)) return false;
         for (auto &inc : addr.slot_incs) {
             if (!base_alignment_ok(inc, prover)) return false;
@@ -430,20 +425,20 @@ struct send_1d_plan_t : public base_plan_t {
             int reg_off, const prover_t &prover) {
         auto &layout = it.parent();
         auto &off = it.block_offset();
-        expr_t addr_inc = layout.offset_in_bytes(off);
+        expr_t addr_inc = layout.shift_in_bytes(off);
         if (!desc.base_alignment_ok(addr_inc, prover)) return false;
         std::vector<expr_t> mask_incs(nmasks());
         auto coord = it.coord();
+        ir_assert(reg_layout.offset_in_bytes(coord) == reg_off);
         for (int i = 0; i < nmasks(); i++) {
             mask_incs[i] = mask_desc[i].to_expr(coord, /*with_const=*/false);
         }
         entries.emplace_back();
         auto &e = entries.back();
-        e.addr_inc = addr_inc;
-        e.mask_incs = mask_incs;
+        e.addr_inc = std::move(addr_inc);
+        e.mask_incs = std::move(mask_incs);
         e.reg_off = reg_off;
-        e.coord = coord;
-        ir_assert(reg_layout.offset_in_bytes(coord) == reg_off);
+        e.coord = std::move(coord);
         return true;
     }
 
@@ -473,6 +468,7 @@ struct send_1d_plan_t : public base_plan_t {
 
 struct send_2d_desc_t {
     hw_t hw;
+    send_address_t address = send_address_t::undef;
     send_op_t op = send_op_t::undef;
     type_t type;
     bool transpose = false;
@@ -499,6 +495,7 @@ struct send_2d_desc_t {
 
         auto &hint = params.hint_2d;
         hw = params.hw;
+        address = params.address;
         op = params.op;
         type = view.type();
         transpose = hint.transpose;
@@ -739,28 +736,30 @@ public:
 
     send_plan_t build() const {
         send_params_t params = init_params_;
+        prb_reqs_t reqs;
+        auto prover = reqs.prover(*params.external_reqs,
+                /*can_update=*/params.kind != send_kind_t::undef);
         switch (params.kind) {
-            case send_kind_t::_2d: return try_build_2d(params, init_view_);
+            case send_kind_t::_2d:
+                return try_build_2d(params, init_view_, prover);
             case send_kind_t::compressed_prefetch: {
-                prb_reqs_t reqs;
                 int cache_line_size = params.hw.cache_line_size();
-                auto view
-                        = init_view_.scatterize(cache_line_size, reqs.prover());
+                auto view = init_view_.scatterize(cache_line_size, prover);
                 if (view.is_empty()) return send_plan_t();
                 params.kind = send_kind_t::scattered;
-                return try_build_1d(params, view, reqs);
+                return try_build_1d(params, view, prover);
             }
-            default: return try_build_1d(params, init_view_);
+            default: return try_build_1d(params, init_view_, prover);
         }
     }
 
 private:
     send_plan_t try_build_1d(const send_params_t &params, const view_t &view,
-            prb_reqs_t reqs = prb_reqs_t()) const {
+            prover_t &prover) const {
         send_plan_t plan(params.hw);
         auto &layout = view.layout();
         auto &mask_desc = view.mask_desc();
-        auto inner_last = find_inner_last(params, view, mask_desc, reqs);
+        auto inner_last = find_inner_last(params, view, mask_desc, prover);
         int type_size = layout.type().size();
         int inner_elems = inner_last.elems();
         int inner_bytes = type_size * inner_elems;
@@ -782,7 +781,7 @@ private:
         int slot_stride = std::max(4, slot_size);
 
         auto inner_end = inner_last + 1;
-        auto middle_last = inner_last;
+        auto middle_last = std::move(inner_last);
         auto outer_begin = end(layout);
         if (is_scattered) {
             // Add blocks to fill up slots in the scattered message.
@@ -802,12 +801,13 @@ private:
 
         send_1d_desc_t desc;
         desc.hw = params.hw;
+        desc.address = params.address;
         desc.op = params.op;
         desc.type_size = slot_size;
         desc.slots = slots;
 
         addr_t addr(layout, slots, elems_per_slot);
-        if (!desc.base_alignment_ok(addr, reqs.prover())) return send_plan_t();
+        if (!desc.base_alignment_ok(addr, prover)) return send_plan_t();
 
         int elem_stride = 1;
         if (slot_stride > slot_size) {
@@ -825,33 +825,32 @@ private:
         auto &plan_1d = plan.get_1d();
         plan_1d = send_1d_plan_t(plan.hw);
         plan_1d.desc = desc;
-        plan_1d.addr = addr;
+        plan_1d.addr = std::move(addr);
         plan_1d.mask = mask_t(mask_desc, layout, slots, elems_per_slot);
-        plan_1d.reg_layout = reg_layout;
-        plan_1d.entry_tile = entry_tile;
+        plan_1d.reg_layout = std::move(reg_layout);
+        plan_1d.entry_tile = std::move(entry_tile);
         for (auto &d : params.skip_mask)
             plan_1d.mask.clear(d);
 
         int step_elems = slots * elems_per_slot;
         layout_iterator_t it(layout);
         int reg_off = 0;
-        plan_1d.add_entry(it, mask_desc, reg_off, reqs.prover());
+        plan_1d.add_entry(it, mask_desc, reg_off, prover);
         while (it.has_next(step_elems)) {
             it.next(step_elems);
             reg_off += slots * slot_stride;
             reg_off = utils::rnd_up(reg_off, grf_size);
-            if (!plan_1d.add_entry(it, mask_desc, reg_off, reqs.prover()))
+            if (!plan_1d.add_entry(it, mask_desc, reg_off, prover))
                 return send_plan_t();
         }
-        plan_1d.reqs = reqs;
-        plan_1d.reqs.simplify();
+        plan_1d.reqs = prover.reqs();
         return plan;
     }
 
     send_plan_t try_build_2d(const send_params_t &params, const view_t &view,
-            prb_reqs_t reqs = prb_reqs_t()) const {
+            prover_t &prover) const {
         send_plan_t plan(params.hw);
-        send_2d_desc_t desc(view, params, reqs.prover());
+        send_2d_desc_t desc(view, params, prover);
         if (!desc) return send_plan_t();
 
         auto &plane = view.plane();
@@ -876,8 +875,8 @@ private:
         plan_2d.mask.clear(plane.y_dim);
         for (auto &d : params.skip_mask)
             plan_2d.mask.clear(d);
-        plan_2d.reg_layout = reg_layout;
-        plan_2d.entry_tile = entry_tile;
+        plan_2d.reg_layout = std::move(reg_layout);
+        plan_2d.entry_tile = std::move(entry_tile);
 
         int reg_off = 0;
         for (int h = 0; h < plane.h; h += desc.h) {
@@ -885,19 +884,18 @@ private:
                 prb_coord_t<int> coord;
                 coord[plane.w_dim] = w;
                 coord[plane.h_dim] = h;
-                if (!plan_2d.add_entry(coord, reg_off, reqs.prover()))
+                if (!plan_2d.add_entry(coord, reg_off, prover))
                     return send_plan_t();
                 reg_off += entry_reg_size;
             }
         }
-        plan_2d.reqs = reqs;
-        plan_2d.reqs.simplify();
+        plan_2d.reqs = prover.reqs();
         return plan;
     }
 
     block_iterator_t find_inner_last(const send_params_t &params,
             const view_t &view, const mask_desc_t &mask_desc,
-            prb_reqs_t &reqs) const {
+            prover_t &prover) const {
         auto &layout = view.layout();
         auto inner_last = begin(layout);
         int type_size = layout.type().size();
@@ -907,8 +905,8 @@ private:
             return type_size * inner_last.elems() >= grf_size;
         };
         for (auto it = begin(layout); it != end(layout); ++it) {
-            auto prover = reqs.prover(!ok_to_return());
-            if (!mask_desc.is_uniform(it, prover)) break;
+            auto _prover = prover_t(prover, /*can_update=*/!ok_to_return());
+            if (!mask_desc.is_uniform(it, _prover)) break;
             if (!it.is_dense()) break;
             if (type_size * it.elems() > params.max_entry_reg_size) break;
             inner_last = it;
