@@ -183,7 +183,7 @@ void layout_raw_tag_t::add_dim(char letter, int pos) {
         if (new_letter >= letter) new_letter++;
         new_entries.emplace_back(new_letter, e.block, e.is_blocked);
     }
-    entries_ = new_entries;
+    entries_ = std::move(new_entries);
 }
 
 void layout_raw_tag_t::remove_dim(char letter) {
@@ -195,7 +195,7 @@ void layout_raw_tag_t::remove_dim(char letter) {
         if (e.letter > letter) new_letter--;
         new_entries.emplace_back(new_letter, e.block, e.is_blocked);
     }
-    entries_ = new_entries;
+    entries_ = std::move(new_entries);
 }
 
 bool layout_raw_tag_t::is_blocked(char letter) const {
@@ -206,7 +206,7 @@ bool layout_raw_tag_t::is_blocked(char letter) const {
 }
 
 int layout_raw_tag_t::ndims() const {
-    ir_assert(!is_any());
+    ir_assert(!is_any() && !has_x());
     int max_index = 0;
     for (auto &e : entries_) {
         max_index = std::max(max_index, e.index());
@@ -257,31 +257,13 @@ bool layout_raw_tag_t::matches(const layout_raw_tag_t &other,
     return i0 == n0 && i1 == n1;
 }
 
-void layout_raw_tag_t::init_entries(const std::string &s) {
-    ir_assert(is_abx_tag(s)) << s;
-    std::array<bool, 'z' - 'a' + 1> is_blocked;
-    is_blocked.fill(false);
-    auto letter_blocks = parse_letter_blocks(s);
-    for (auto &p : letter_blocks) {
-        if (p.second != 0) is_blocked[std::tolower(p.first) - 'a'] = true;
-    }
-    for (auto &p : letter_blocks) {
-        char letter = std::tolower(p.first);
-        entries_.emplace_back();
-        auto &e = entries_.back();
-        e.letter = letter;
-        e.block = p.second;
-        e.is_blocked = is_blocked[letter - 'a'];
-    }
-}
-
 bool layout_raw_tag_t::has_x() const {
     for (auto &e : entries_)
         if (e.is_x()) return true;
     return false;
 }
 
-void layout_raw_tag_t::normalize(int ndims) {
+void layout_raw_tag_t::expand_x(int ndims) {
     if (!has_x() || ndims == 0) return;
     std::vector<layout_raw_tag_entry_t> new_entries;
     for (auto &e : entries_) {
@@ -295,7 +277,63 @@ void layout_raw_tag_t::normalize(int ndims) {
             new_entries.push_back(e);
         }
     }
-    entries_ = new_entries;
+    entries_ = std::move(new_entries);
+}
+
+layout_raw_tag_t layout_raw_tag_t::collapse_x() const {
+    if (has_x()) return *this;
+    auto tag = to_tag(entries_);
+    for (int i = 2; i < ndims() - 1; i++) {
+        std::string x_expanded;
+        bool ok = true;
+        for (int j = i; j < ndims(); j++) {
+            for (auto &e : entries_) {
+                if (e.index() == j && e.is_blocked) {
+                    ok = false;
+                    break;
+                }
+            }
+            x_expanded += 'a' + j;
+        }
+        if (!ok) continue;
+        auto pos = tag.find(x_expanded);
+        if (pos == std::string::npos) continue;
+        tag.replace(pos, x_expanded.length(), "x");
+        return layout_raw_tag_t(tag);
+    }
+    return *this;
+}
+
+std::vector<layout_raw_tag_entry_t> layout_raw_tag_t::to_entries(
+        const std::string &tag) {
+    if (tag == "any") return {};
+    ir_assert(is_abx_tag(tag)) << tag;
+    std::array<bool, 'z' - 'a' + 1> is_blocked;
+    is_blocked.fill(false);
+    auto letter_blocks = parse_letter_blocks(tag);
+    for (auto &p : letter_blocks) {
+        if (p.second != 0) is_blocked[std::tolower(p.first) - 'a'] = true;
+    }
+    std::vector<layout_raw_tag_entry_t> entries;
+    for (auto &p : letter_blocks) {
+        char letter = std::tolower(p.first);
+        entries.emplace_back();
+        auto &e = entries.back();
+        e.letter = letter;
+        e.block = p.second;
+        e.is_blocked = is_blocked[letter - 'a'];
+    }
+    return entries;
+}
+
+std::string layout_raw_tag_t::to_tag(
+        const std::vector<layout_raw_tag_entry_t> &entries) {
+    std::string tag;
+    for (auto &e : entries) {
+        if (e.is_blocked) { tag += std::to_string(e.block); }
+        tag += e.letter;
+    }
+    return tag;
 }
 
 std::vector<bool> layout_raw_tag_t::skip_mask(
@@ -381,6 +419,7 @@ bool layout_tag_t::matches(
 }
 
 std::string layout_tag_t::str() const {
+    if (is_empty()) return "x";
     std::ostringstream oss;
     oss << raw_tag_ << ":" << type_;
     return oss.str();
@@ -472,7 +511,7 @@ expr_t layout_t::stride(const prb_dim_t &dim, int dim_block_idx) const {
     return expr_t();
 }
 
-expr_t layout_t::offset_in_bytes(const std::vector<int> &block_off) const {
+expr_t layout_t::shift_in_bytes(const std::vector<int> &block_off) const {
     expr_t ret = 0;
     for (int i = 0; i < nblocks(); i++) {
         auto &b = blocks_[i];
@@ -483,8 +522,7 @@ expr_t layout_t::offset_in_bytes(const std::vector<int> &block_off) const {
 
 int layout_t::offset_in_bytes(prb_coord_t<int> coord) const {
     ir_assert(has_const_sizes() && has_const_strides());
-    ir_assert(has_zero_base());
-    int ret = 0;
+    int ret = to_cpp<int>(base_);
     for (int i = 0; i < nblocks(); i++) {
         auto &b = blocks_[i];
         int &rem_dim = coord[b.dim];
@@ -541,6 +579,18 @@ void layout_t::add_block(
     blocks_.emplace_back(dim, size, stride);
 }
 
+void layout_t::remove(const prb_dim_t &dim) {
+    std::vector<block_t> new_blocks;
+    for (auto &b : blocks_) {
+        if (b.dim == dim) continue;
+        new_blocks.push_back(b);
+    }
+    auto new_letter_map = desc_.letter_map();
+    new_letter_map.unset(dim);
+    desc_ = layout_desc_t(new_letter_map);
+    blocks_ = std::move(new_blocks);
+}
+
 void layout_t::block_by(const std::vector<block_t> &inner_blocks) {
     ir_assert(has_zero_base());
     ir_assert(has_const_sizes());
@@ -614,18 +664,54 @@ layout_t layout_t::split_block(
 }
 
 template <typename T>
-struct div_helper_t {
-    static T call(const T &a, int b) { return a / b; }
+struct try_div_mod {
+    static bool call(const T &a, int b, const var_range_info_t &range_info,
+            T &div, T &mod) {
+        if (a % b != 0) return false;
+        div = a / b;
+        mod = a % b;
+        return true;
+    }
 };
 
 template <>
-struct div_helper_t<expr_t> {
-    static expr_t call(const expr_t &a, int b) { return linear_div(a, b); }
+struct try_div_mod<expr_t> {
+    static bool call(const expr_t &a, int b, const var_range_info_t &range_info,
+            expr_t &div, expr_t &mod) {
+        int factor = linear_max_pow2_divisor(a);
+        if (factor % b == 0) {
+            div = linear_div(a, b);
+            mod = expr_t(0);
+            return true;
+        }
+        auto _linear = to_linear(a);
+        auto &linear = _linear.as<linear_t>();
+        int c_factor = linear_max_pow2_divisor(linear.c);
+        if (c_factor % b != 0) return false;
+        expr_t a_div = linear_div(linear.c, b);
+        expr_t a_mod;
+        for (int i = 0; i < linear.nargs(); i++) {
+            auto &u = linear.u_vec[i];
+            auto &v = linear.v_vec[i];
+            int u_factor = linear_max_pow2_divisor(u);
+            if (u_factor % b == 0) {
+                a_div += linear_div(u, b) * v;
+                continue;
+            }
+            if (range_info.bound(v) > b) return false;
+            if (!a_mod.is_empty()) return false;
+            a_mod = v;
+        }
+        div = a_div;
+        mod = a_mod;
+        return true;
+    }
 };
 
 template <typename T>
 layout_t layout_t::map(const dim_mapper_t &dim_mapper,
-        const prb_coord_t<T> &coord, const prb_tile_t &tile) const {
+        const prb_coord_t<T> &coord, const prb_tile_t &tile,
+        const var_range_info_t &var_range_info) const {
     auto idxs = coord;
     auto rem_sizes = tile;
     idxs.fill_missing(0);
@@ -650,7 +736,7 @@ layout_t layout_t::map(const dim_mapper_t &dim_mapper,
                         int inner = cur_size;
                         int outer = b_size / cur_size;
                         return split_block(&b, inner, outer)
-                                .map(dim_mapper, coord, tile);
+                                .map(dim_mapper, coord, tile, var_range_info);
                     }
                     return layout_t();
                 }
@@ -663,10 +749,14 @@ layout_t layout_t::map(const dim_mapper_t &dim_mapper,
             }
             bool is_outer = true;
             if (b.has_const_size()) {
+                ir_assert(is_zero(off));
                 ir_assert(!seen_outer.has(dim));
-                int factor = linear_max_pow2_divisor(idxs[dim]);
-                if (factor % b.int_size() == 0) {
-                    idxs[dim] = div_helper_t<T>::call(idxs[dim], b.int_size());
+                T div = T();
+                T mod = T();
+                if (try_div_mod<T>::call(idxs[dim], b.int_size(),
+                            var_range_info, div, mod)) {
+                    idxs[dim] = div;
+                    off = mod;
                     is_outer = false;
                 }
             }
@@ -682,9 +772,11 @@ layout_t layout_t::map(const dim_mapper_t &dim_mapper,
 }
 
 template layout_t layout_t::map<int>(const dim_mapper_t &dim_mapper,
-        const prb_coord_t<int> &coord, const prb_tile_t &tile) const;
+        const prb_coord_t<int> &coord, const prb_tile_t &tile,
+        const var_range_info_t &var_range_info) const;
 template layout_t layout_t::map<expr_t>(const dim_mapper_t &dim_mapper,
-        const prb_coord_t<expr_t> &coord, const prb_tile_t &tile) const;
+        const prb_coord_t<expr_t> &coord, const prb_tile_t &tile,
+        const var_range_info_t &var_range_info) const;
 
 prb_coord_t<int> layout_t::to_coord(const std::vector<int> &block_idx) const {
     ir_assert((int)block_idx.size() == nblocks());
@@ -974,7 +1066,7 @@ dim_mask_desc_t dim_mask_desc_t::map(const prb_coord_t<expr_t> &coord) const {
     auto ret = *this;
     ret.base = simplify_rewrite(to_expr(coord));
     if (!is_identity()) return ret;
-    int x_div = linear_max_pow2_divisor(coord[x_dim]);
+    int x_div = linear_max_pow2_divisor(coord.get(x_dim, 0));
     ret.block = math::gcd(block, x_div);
     return ret;
 }
@@ -1142,8 +1234,9 @@ bool grid_splitter_t::is_empty() const {
     return true;
 }
 
-expr_t grid_splitter_t::pop(int size) {
+expr_t grid_splitter_t::pop(int _size) {
     expr_t cur = 0;
+    int size = _size;
     for (auto &idx : idxs_) {
         if (idx.size == 1) continue;
         if (size == 1) break;
@@ -1151,14 +1244,14 @@ expr_t grid_splitter_t::pop(int size) {
         cur += idx.pop(size);
     }
     ir_assert(size == 1);
-    return register_index(simplify_rewrite(cur));
+    return register_index(simplify_rewrite(cur), _size);
 }
 
 expr_t grid_splitter_t::index_t::pop(int &n) {
     if (n == 1) return 0;
     if (size >= n) {
         ir_assert(size % n == 0);
-        auto ret = expr % n;
+        auto ret = (size == n ? expr : expr % n);
         expr = (size == n ? 0 : expr / n);
         size /= n;
         n = 1;
@@ -1172,23 +1265,25 @@ expr_t grid_splitter_t::index_t::pop(int &n) {
     return ret;
 }
 
-expr_t grid_splitter_t::register_index(const expr_t &expr) {
+expr_t grid_splitter_t::register_index(const expr_t &expr, int size) {
     if (expr.is<var_t>()) return expr;
     int idx = (int)virt_grid_idxs_.size();
     auto var
             = var_t::make(type_t::s32(), "virt_grid_idx" + std::to_string(idx));
     virt_grid_idxs_.emplace(var, expr);
+    var_range_info_.set_bound(var, size);
     return var;
 }
 
 view_t::view_t(const dim_mapper_t &dim_mapper, const layout_t &base_layout,
-        const prb_coord_t<expr_t> &coord, const prb_tile_t &tile)
+        const prb_coord_t<expr_t> &coord, const prb_tile_t &tile,
+        const var_range_info_t &var_range_info)
     : dim_mapper_(dim_mapper)
     , base_layout_(base_layout)
     , coord_(coord)
     , tile_(tile) {
     mask_desc_t base_mask_desc(dim_mapper, base_layout);
-    layout_ = base_layout.map(dim_mapper, coord, tile);
+    layout_ = base_layout.map(dim_mapper, coord, tile, var_range_info);
     mask_desc_ = base_mask_desc.map(coord);
     plane_ = plane_t(layout_, mask_desc_);
 }
@@ -1311,6 +1406,7 @@ view_t view_t::split(const dim_mapper_t &dim_mapper,
             int b_size = b.int_size();
             split_tile[b.dim]
                     = ir_utils::safe_div(split_tile.at(b.dim), b_size);
+            if (!split_coord.has(b.dim)) split_coord[b.dim] = expr_t(0);
             split_coord[b.dim]
                     += grid_splitter.pop(b_size) * inner_dims.at(b.dim);
             split_coord[b.dim] = simplify_rewrite(split_coord[b.dim]);
@@ -1318,7 +1414,8 @@ view_t view_t::split(const dim_mapper_t &dim_mapper,
         inner_dims[b.dim] *= b.int_size();
     }
     ir_assert(grid_splitter.is_empty());
-    return view_t(dim_mapper, base_layout, split_coord, split_tile);
+    return view_t(dim_mapper, base_layout, split_coord, split_tile,
+            grid_splitter.var_range_info());
 }
 
 } // namespace v2
