@@ -32,6 +32,7 @@ using namespace bn_lookup_table;
 using namespace bn_utils;
 using namespace bn_model;
 using namespace dnnl::impl::utils;
+using namespace dnnl::impl::gpu::intel::gpu_utils;
 
 static size_t get_slm_buff_size(
         int ic_block, nhwc_bnorm_params_t &conf, const compute::range_t &lws) {
@@ -47,7 +48,7 @@ static size_t get_slm_buff_size(
 }
 // Local group size adjustment for calc_stat kernel
 static void adjust_lws_calc_kernel(int ic_block, nhwc_bnorm_params_t &conf,
-        compute::dispatch_t &dispatch, engine_t *engine,
+        compute::dispatch_t &dispatch, impl::engine_t *engine,
         bool large_grf_mode = false) {
     auto *compute_engine = downcast<compute::compute_engine_t *>(engine);
     auto eu_count = compute_engine->device_info()->eu_count();
@@ -109,7 +110,7 @@ static int get_reduce_sub_group_count(
 }
 
 status_t nhwc_bnorm_kernel_dispatching(kernel_kind_t kernel,
-        nhwc_bnorm_params_t &conf, engine_t *engine,
+        nhwc_bnorm_params_t &conf, impl::engine_t *engine,
         compute::dispatch_t &dispatch) {
 
     conf.stat_sp_nblocks
@@ -193,11 +194,12 @@ static status_t init_conf_common(nhwc_bnorm_params_t &conf, offsets_t &off,
         compute::dispatch_t &dispatch_calc_stat,
         compute::dispatch_t &dispatch_reduce_stat,
         compute::dispatch_t &dispatch, compute::dispatch_t &dispatch_reduce_aux,
-        const batch_normalization_pd_t *pd, engine_t *engine) {
+        const batch_normalization_pd_t *pd, impl::engine_t *engine) {
     using namespace dnnl::impl::format_tag;
     const memory_desc_wrapper data_mdw(
             pd->is_fwd() ? pd->src_md() : pd->diff_src_md());
 
+    conf.impl = bn_impl_t::nhwc_opt;
     init_conf_basic(conf, pd);
     set_offsets(data_mdw, off.src_off);
 
@@ -365,7 +367,8 @@ static status_t init_kernel_ctx_common(compute::kernel_ctx_t &kernel_ctx,
     return status::success;
 }
 
-status_t nhwc_batch_normalization_fwd_t::pd_t::init_conf(engine_t *engine) {
+status_t nhwc_batch_normalization_fwd_t::pd_t::init_conf(
+        impl::engine_t *engine) {
     return init_conf_common(conf, off, dispatch_calc_stat, dispatch_reduce_stat,
             dispatch, dispatch_reduce_aux, this, engine);
 }
@@ -568,7 +571,8 @@ status_t nhwc_batch_normalization_fwd_t::execute_forward(
     return status;
 }
 
-status_t nhwc_batch_normalization_bwd_t::pd_t::init_conf(engine_t *engine) {
+status_t nhwc_batch_normalization_bwd_t::pd_t::init_conf(
+        impl::engine_t *engine) {
     return init_conf_common(conf, off, dispatch_calc_stat, dispatch_reduce_stat,
             dispatch, dispatch_reduce_aux, this, engine);
 }
