@@ -236,7 +236,7 @@ bool fits_xab_xba(const memory_desc_wrapper &src,
 }
 
 bool matches_ABxxxx8ayb_layout(const blocking_desc_t &blk, int ndims) {
-    if (ndims > 2) { return false; }
+    if (ndims > 2 || blk.inner_nblks < 2) { return false; }
     int last = blk.inner_nblks - 1;
     // Don't allow this kernel when two adjacent blocks by b create
     // total block size smaller than 16 - in that situation macros
@@ -528,7 +528,7 @@ void custom_reorder_t::pd_t::alt_gen() {
     conf.dispatch.generate_override(gws, lws);
 }
 
-status_t custom_reorder_t::pd_t::init_conf(engine_t *engine) {
+status_t custom_reorder_t::pd_t::init_conf(impl::engine_t *engine) {
     using namespace format_tag;
 
     const memory_desc_wrapper src_mdw(src_md());
