@@ -17,6 +17,7 @@
 #ifndef GPU_INTEL_OCL_OCL_MATH_UTILS_H
 #define GPU_INTEL_OCL_OCL_MATH_UTILS_H
 
+#include "gpu/intel/ocl/dispatch.h"
 #include "gpu/intel/ocl/ocl_utils.h"
 
 // Due to JIT compilation and a lack of bitwise operations in implementations,
@@ -36,27 +37,29 @@ int rnd_down(int a, unsigned int b) {
 #pragma OPENCL EXTENSION cl_khr_fp16 : enable
 
 #if DT_BF8 || SRC_DT_BF8 || WEI_DT_BF8 || DST_DT_BF8 || BIA_DT_BF8 || A_DT_BF8 \
-        || B_DT_BF8 || C_DT_BF8
+        || B_DT_BF8 || C_DT_BF8 || DATA_DT_BF8 || POST_OP_USING_BF8
 #define MATH_UTILS_DECLARE_BF8 1
 #endif
 
 #if DT_HF8 || SRC_DT_HF8 || WEI_DT_HF8 || DST_DT_HF8 || BIA_DT_HF8 || A_DT_HF8 \
-        || A_DT_HF8 || B_DT_HF8 || C_DT_HF8
+        || A_DT_HF8 || B_DT_HF8 || C_DT_HF8 || DATA_DT_HF8 \
+        || POST_OP_USING_HF8
 #define MATH_UTILS_DECLARE_HF8 1
 #endif
 
 #if DT_S4 || SRC_DT_S4 || WEI_DT_S4 || DST_DT_S4 || BIA_DT_S4 || A_DT_S4 \
-        || B_DT_S4 || C_DT_S4
+        || B_DT_S4 || C_DT_S4 || DATA_DT_S4 || WEI_ZP_DT_S4 || SRC_ZP_DT_S4
 #define MATH_UTILS_DECLARE_S4 1
 #endif
 
 #if DT_U4 || SRC_DT_U4 || WEI_DT_U4 || DST_DT_U4 || BIA_DT_U4 || A_DT_U4 \
-        || A_DT_U4 || B_DT_U4 || C_DT_U4
+        || A_DT_U4 || B_DT_U4 || C_DT_U4 || DATA_DT_U4 || WEI_ZP_DT_U4 \
+        || SRC_ZP_DT_U4
 #define MATH_UTILS_DECLARE_U4 1
 #endif
 
 #if DT_BF16 || SRC_DT_BF16 || WEI_DT_BF16 || DST_DT_BF16 || BIA_DT_BF16 \
-        || A_DT_BF16 || B_DT_BF16 || C_DT_BF16 || SUM_DT_BF16 \
+        || A_DT_BF16 || B_DT_BF16 || C_DT_BF16 || SUM_DT_BF16 || DATA_DT_BF16 \
         || POST_OP_USING_BF16
 #define MATH_UTILS_DECLARE_BF16 1
 #endif
@@ -66,6 +69,11 @@ ushort16 __builtin_IB_simd_block_read_16_global_h(const __global ushort *);
 
 void __builtin_IB_simd_block_write_8_global_l(__global ulong *, ulong8);
 void __builtin_IB_simd_block_write_16_global_h(__global ushort *, ushort16);
+float __attribute__((overloadable)) cvt_e8m0_to_f32(uchar f) {
+    if (f == (uchar)0xff) return as_float(0xffc00000);
+    uint bits = f << 23;
+    return as_float(bits);
+}
 
 #if MATH_UTILS_DECLARE_HF8
 // Emulation functions for f8_e4m3 <-> f16 conversion.
@@ -661,15 +669,21 @@ DECLARE_MMAD_EMU(mmad8x8_bf16, bf16_dot2, 8, 8, short8, int8, float8)
 #endif
 
 // Atomics
-#if !DETERMINISTIC
+#if !DETERMINISTIC && ATOMICS_SUPPORTED
+#define ATOMIC(x) CONCAT2(atomic_, x)
 #define DECLARE_ATOMIC_OP(op, type) \
     type __attribute__((overloadable)) CONCAT3(atomic_, op, _global)( \
-            volatile global CONCAT2(atomic_, type) * source, type operand) { \
+            volatile global ATOMIC(type) * source, type operand) { \
         return CONCAT3(atomic_fetch_, op, _explicit)( \
                 source, operand, memory_order_relaxed); \
     }
 
-#ifdef cl_ext_float_atomics
+DECLARE_ATOMIC_OP(min, int)
+DECLARE_ATOMIC_OP(max, int)
+DECLARE_ATOMIC_OP(add, int)
+DECLARE_ATOMIC_OP(sub, int)
+
+#if ATOMIC_FLOAT_SUPPORTED
 #ifdef __opencl_c_ext_fp32_global_atomic_add
 #define HAS_FLOAT_ATOMIC_ADD
 DECLARE_ATOMIC_OP(add, float)
@@ -680,10 +694,10 @@ DECLARE_ATOMIC_OP(sub, float)
 DECLARE_ATOMIC_OP(min, float)
 DECLARE_ATOMIC_OP(max, float)
 #endif // __opencl_c_ext_fp32_global_atomic_min_max
+#endif // ATOMIC_FLOATS_SUPPORTED
 
-#endif // cl_ext_float_atomics
-
-#if __OPENCL_C_VERSION__ >= 200 && !defined(HAS_FLOAT_ATOMIC_ADD)
+#ifndef HAS_FLOAT_ATOMIC_ADD
+// Fallback atomic implementations
 inline float atomic_add_global(
         volatile __global atomic_float *source, float operand) {
     float old_val = atomic_load_explicit(
@@ -701,6 +715,7 @@ inline float atomic_add_global(
 #endif
 
 #if MATH_UTILS_DECLARE_S4 || MATH_UTILS_DECLARE_U4
+#define GET_HALF_BYTE(x, y) get_half_byte(x, y)
 
 uchar __attribute__((overloadable)) cvt_f32_to_u4(float a) {
     uchar i = convert_uchar_sat_rte(a);
@@ -717,6 +732,12 @@ float __attribute__((overloadable)) cvt_s4_to_f32(char a) {
     return convert_float(val);
 }
 
+float __attribute__((overloadable)) cvt_s4_to_s32(char a) {
+    char sign = (a & 0x08) ? 0xf0 : 0x0;
+    char val = a | sign;
+    return convert_int_sat_rte(val);
+}
+
 uchar __attribute__((overloadable)) get_half_byte(__global uchar *x, off_t y) {
     uchar ret = 0;
     if (y % 2) {
