@@ -26,6 +26,7 @@
 #include "common/primitive_exec_types.hpp"
 #include "common/utils.hpp"
 #include "gpu/gpu_eltwise_pd.hpp"
+#include "gpu/gpu_utils.hpp"
 #include "gpu/intel/block_structure.hpp"
 #include "gpu/intel/compute/dispatch.hpp"
 #include "gpu/intel/compute/kernel_arg_list.hpp"
@@ -48,18 +49,6 @@ bool memory_desc_ndims_ok(const T *first, const Rest *...rest) {
     return memory_desc_ndims_ok(first) || memory_desc_ndims_ok(rest...);
 }
 
-inline dim_t get_attr_oscales_count(int mask, const memory_desc_wrapper &md) {
-    dim_t count = 1;
-    if (mask == 0) return count;
-
-    for (int d = 0; d < md.ndims(); d++) {
-        const int dim_mask = 1 << d;
-        if (dim_mask & mask) count *= md.dims()[d];
-    }
-
-    return count;
-}
-
 struct memory_desc_info_t {
     // Max levels of blocking
     static const int max_nlevels = 3;
@@ -163,18 +152,15 @@ struct attr_info_t {
 
         attr_info.with_runtime_oscales = !attr->output_scales_.defined();
 
-        const auto &src0_scales = attr->scales_.get(DNNL_ARG_SRC_0);
-        attr_info.with_src0_scale = !src0_scales.has_default_values();
-        gpu_assert(src0_scales.mask_ == 0);
+        const auto &src_scales = attr->scales_.get(DNNL_ARG_SRC);
+        attr_info.with_src_scales = !src_scales.has_default_values();
+        attr_info.with_src0_scale = !src_scales.has_default_values();
+        attr_info.src_scales_mask = src_scales.mask_;
 
         const auto &src1_scales = attr->scales_.get(DNNL_ARG_SRC_1);
         attr_info.with_src1_scale = !src1_scales.has_default_values();
         gpu_assert(src1_scales.mask_ == 0);
 
-        const auto &src_scales = attr->scales_.get(DNNL_ARG_SRC);
-        attr_info.with_src_scales = !src_scales.has_default_values();
-        gpu_assert(src_scales.mask_ == 0);
-
         const auto &wei_scales = attr->scales_.get(DNNL_ARG_WEIGHTS);
         attr_info.with_wei_scales = !wei_scales.has_default_values();
         attr_info.wei_scales_mask = wei_scales.mask_;
@@ -188,6 +174,9 @@ struct attr_info_t {
         attr_info.with_src_zpoints = !zp.has_default_values(DNNL_ARG_SRC);
         attr_info.with_wei_zpoints = !zp.has_default_values(DNNL_ARG_WEIGHTS);
         attr_info.with_dst_zpoints = !zp.has_default_values(DNNL_ARG_DST);
+        attr_info.src_zpoints_data_type = zp.get_data_type(DNNL_ARG_SRC);
+        attr_info.wei_zpoints_data_type = zp.get_data_type(DNNL_ARG_WEIGHTS);
+        attr_info.dst_zpoints_data_type = zp.get_data_type(DNNL_ARG_DST);
 
         attr_info.with_per_ic_src_zpoints = attr_info.with_src_zpoints
                 && !zp.defined(DNNL_ARG_SRC) && !zp.common(DNNL_ARG_SRC);
@@ -225,6 +214,7 @@ struct attr_info_t {
     bool with_src_scales;
     bool with_wei_scales;
     bool with_dst_scales;
+    bool src_scales_mask;
     bool wei_scales_mask;
 
     bool with_src_zpoints;
@@ -232,6 +222,9 @@ struct attr_info_t {
     bool with_dst_zpoints;
     bool with_per_ic_src_zpoints;
     bool with_per_oc_dst_zpoints;
+    data_type_t src_zpoints_data_type;
+    data_type_t wei_zpoints_data_type;
+    data_type_t dst_zpoints_data_type;
 };
 
 template <size_t ndims>
@@ -460,6 +453,16 @@ struct rnn_reorder_conf_t {
 };
 
 // Batch Normalization
+enum bn_impl_t {
+    unknown = 0,
+    ref,
+    simple,
+    reusable,
+    gen9,
+    nhwc_opt,
+    nhwc_reusable
+};
+
 struct bnorm_conf_t {
     data_type_t data_type;
     size_t elsz;
@@ -485,6 +488,8 @@ struct bnorm_conf_t {
     bool skip_reduce_stat;
     bool use_stats_one_pass;
     int calc_stat_ic;
+    int max_ic_block;
+    bn_impl_t impl = bn_impl_t::unknown;
 };
 
 // Layer Normalization
@@ -667,71 +672,9 @@ union reorder_implementation {
     vectorize_last_dim_t vld;
 };
 
-class scales_query_t {
+struct quantization_t : public gpu::quantization_t {
 public:
-    bool has_default_values() const { return scales_.has_default_values(); }
-    int get_mask() const { return scales_.mask_; }
-    size_t get_count() const { return count_; }
-    memory_storage_t &get_scales(const exec_ctx_t &ctx) const {
-        return CTX_IN_STORAGE(DNNL_ARG_ATTR_SCALES | arg_);
-    }
-
-    scales_query_t() = default;
-    scales_query_t(const primitive_attr_t *attr, const memory_desc_wrapper &mdw,
-            int arg)
-        : arg_(arg) {
-        scales_ = attr->scales_.get(arg);
-        count_ = get_attr_oscales_count(scales_.mask_, mdw);
-    }
-
-private:
-    runtime_scales_t scales_;
-    dim_t count_ = 0;
-    int arg_ = 0;
-};
-
-class zero_points_query_t {
-public:
-    bool has_default_values() const { return zps_.has_default_values(arg_); }
-    int get_mask() const {
-        int mask = zps_.get(arg_);
-        return mask;
-    }
-    size_t get_count() const { return count_; }
-    memory_storage_t &get_zero_points(const exec_ctx_t &ctx) const {
-        return CTX_IN_STORAGE(DNNL_ARG_ATTR_ZERO_POINTS | arg_);
-    }
-
-    zero_points_query_t() = default;
-    zero_points_query_t(const primitive_attr_t *attr,
-            const memory_desc_wrapper &mdw, int arg)
-        : arg_(arg) {
-        zps_ = attr->zero_points_;
-        int mask = zps_.get(arg);
-        count_ = get_attr_oscales_count(mask, mdw);
-    }
-
-private:
-    zero_points_t zps_;
-    dim_t count_ = 0;
-    int arg_ = 0;
-};
-
-struct quantization_t {
-public:
-    bool with_scale() const { return !scale_.has_default_values(); }
-    int scale_mask() const { return scale_.get_mask(); }
-    size_t num_scales() const { return scale_.get_count(); }
-    memory_storage_t &scales(const exec_ctx_t &ctx) const {
-        return scale_.get_scales(ctx);
-    }
-
-    bool with_zp() const { return !zp_.has_default_values(); }
-    int zp_mask() const { return zp_.get_mask(); }
-    size_t num_zps() const { return zp_.get_count(); }
-    memory_storage_t &zero_points(const exec_ctx_t &ctx) const {
-        return zp_.get_zero_points(ctx);
-    }
+    using gpu::quantization_t::quantization_t;
 
     void define_macros(
             compute::kernel_ctx_t &kernel_ctx, const std::string &name) const {
@@ -747,49 +690,17 @@ public:
             kernel_ctx.define_int(name + "_NUM_ZPOINTS", num_zps());
         }
     }
-
-    quantization_t(const primitive_attr_t *attr, const memory_desc_wrapper &mdw,
-            int arg)
-        : scale_(attr, mdw, arg), zp_(attr, mdw, arg) {}
-    quantization_t() = default;
-
-private:
-    scales_query_t scale_;
-    zero_points_query_t zp_;
 };
 
-struct sum_quantization_t {
+struct sum_quantization_t : public gpu::sum_quantization_t {
 public:
-    bool with_scale() const { return scale_ != 0; }
-    int scale_mask() const { return 0; }
-    size_t num_scales() const { return (size_t)(with_scale()); }
-    float scales() const { return scale_; }
-
-    bool with_zp() const { return zp_ != 0; }
-    int zp_mask() const { return 0; }
-    size_t num_zps() const { return (size_t)(with_zp()); }
-    int zero_points() const { return zp_; }
+    using gpu::sum_quantization_t::sum_quantization_t;
 
     void define_macros(
             compute::kernel_ctx_t &kernel_ctx, const std::string &name) const {
         if (with_scale()) kernel_ctx.define_int("WITH_" + name + "_SCALE", 1);
         if (with_zp()) kernel_ctx.define_int("WITH_" + name + "_ZPOINT", 1);
     }
-
-    sum_quantization_t(const primitive_attr_t *attr) {
-        const auto &post_ops = attr->post_ops_;
-        const int sum_idx = post_ops.find(primitive_kind::sum);
-        if (sum_idx != -1) {
-            const auto &sum = post_ops.entry_[sum_idx].sum;
-            scale_ = sum.scale;
-            zp_ = sum.zero_point;
-        }
-    }
-    sum_quantization_t() = default;
-
-private:
-    float scale_ = 0;
-    int zp_ = 0;
 };
 
 struct reorder_conf_t {
@@ -801,6 +712,7 @@ struct reorder_conf_t {
     reorder_kernel_t implementation;
     int ndims;
     size_t nelems;
+    bool subbyte_pack = false;
 
     compute::dispatch_t dispatch;
 
@@ -844,6 +756,7 @@ struct concat_conf_t {
     int iter_dim_idx, iter_dim_chunk;
     scales_query_t scale_src[64];
     uint64_t scales_mask;
+    bool use_large_index = true;
 };
 
 // Shuffle
@@ -1091,12 +1004,23 @@ inline void def_block_offsets(const block_layout_t &layout,
     }
 }
 
-inline void def_data_type(
-        compute::kernel_ctx_t &kernel_ctx, data_type_t dt, const char *str) {
+inline void def_data_type(compute::kernel_ctx_t &kernel_ctx, data_type_t dt,
+        const char *str, bool with_punning = true) {
+    const char *bf16_name = with_punning ? "ushort" : "bf16";
+    const char *bf8_name = with_punning ? "uchar" : "f8_e5m2";
+    const char *hf8_name = with_punning ? "uchar" : "f8_e4m3";
+    const char *e8m0_name = with_punning ? "uchar" : "e8m0";
+    const char *u4_name = with_punning ? "uchar" : "u4";
+    const char *s4_name = with_punning ? "uchar" : "s4";
+
     switch (dt) {
-        case data_type::bf16:
+        case data_type::undef:
             kernel_ctx.add_option(
-                    utils::format("-D%s_DATA_T=ushort -D%s_DT_BF16", str, str));
+                    utils::format("-D%s_DATA_T=void -D%s_DT_UNDEF", str, str));
+            break;
+        case data_type::bf16:
+            kernel_ctx.add_option(utils::format(
+                    "-D%s_DATA_T=%s -D%s_DT_BF16", str, bf16_name, str));
             break;
         case data_type::f16:
             kernel_ctx.add_option(
@@ -1119,26 +1043,29 @@ inline void def_data_type(
                     utils::format("-D%s_DATA_T=uchar -D%s_DT_U8", str, str));
             break;
         case data_type::f8_e4m3:
-            kernel_ctx.add_option(
-                    utils::format("-D%s_DATA_T=uchar -D%s_DT_HF8", str, str));
+            kernel_ctx.add_option(utils::format(
+                    "-D%s_DATA_T=%s -D%s_DT_HF8", str, hf8_name, str));
             break;
         case data_type::f8_e5m2:
-            kernel_ctx.add_option(
-                    utils::format("-D%s_DATA_T=uchar -D%s_DT_BF8", str, str));
+            kernel_ctx.add_option(utils::format(
+                    "-D%s_DATA_T=%s -D%s_DT_BF8", str, bf8_name, str));
+            break;
+        case data_type::e8m0:
+            kernel_ctx.add_option(utils::format(
+                    "-D%s_DATA_T=%s -D%s_DT_E8M0", str, e8m0_name, str));
             break;
         case data_type::s4:
-            kernel_ctx.add_option(
-                    utils::format("-D%s_DATA_T=char -D%s_DT_S4", str, str));
+            kernel_ctx.add_option(utils::format(
+                    "-D%s_DATA_T=%s -D%s_DT_S4", str, s4_name, str));
             break;
         case data_type::u4:
-            kernel_ctx.add_option(
-                    utils::format("-D%s_DATA_T=uchar -D%s_DT_U4", str, str));
+            kernel_ctx.add_option(utils::format(
+                    "-D%s_DATA_T=%s -D%s_DT_U4", str, u4_name, str));
             break;
         case data_type::s32:
             kernel_ctx.add_option(
                     utils::format("-D%s_DATA_T=int -D%s_DT_S32", str, str));
             break;
-        case data_type::undef: break;
         default:
             gpu_error_not_expected()
                     << "Unexpected data type " << dnnl_dt2str(dt);
@@ -1304,8 +1231,19 @@ inline status_t def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
     std::string po_kernel_args = "-DPOST_OP_ARGS=\"";
     int nof_supported_post_ops = 0;
 
+    bool post_op_uses_bf16 = false;
+    bool post_op_uses_bf8 = false;
+    bool post_op_uses_hf8 = false;
+
     auto add_po_defines = [&](const std::string &bin_arg_name,
                                   const post_ops_t::entry_t &e, int idx) {
+        auto define_binary_po_type = [&](data_type_t type) {
+            def_data_type(kernel_ctx, type,
+                    utils::format("%s_ACTUAL", bin_arg_name).c_str(), false);
+            post_op_uses_bf16 |= (type == data_type::bf16);
+            post_op_uses_bf8 |= (type == data_type::f8_e5m2);
+            post_op_uses_hf8 |= (type == data_type::f8_e4m3);
+        };
         if (e.is_binary()) {
             kernel_ctx.define_int(
                     "PO_" + std::to_string(idx) + "_KIND", po_binary_id);
@@ -1315,13 +1253,7 @@ inline status_t def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
             const memory_desc_wrapper src1_mdw(e.binary.src1_desc);
             const auto mdi = memory_desc_info_t::create(src1_mdw);
             def_memory_desc_info(kernel_ctx, mdi, bin_arg_name.c_str());
-            if (mdi.data_type == data_type::bf16) {
-                kernel_ctx.define_int(
-                        "PO_" + std::to_string(idx) + "_BIN_ARG_DT_IS_BF16", 1);
-            } else {
-                kernel_ctx.define_int(
-                        "PO_" + std::to_string(idx) + "_BIN_ARG_DT_IS_BF16", 0);
-            }
+            define_binary_po_type(mdi.data_type);
         } else if (e.is_prelu()) {
             // binary && eltwise relu = prelu post op
             kernel_ctx.define_int(
@@ -1338,8 +1270,7 @@ inline status_t def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
             def_memory_desc_info(kernel_ctx, mdi, bin_arg_name.c_str());
 
             // prelu weights are assumed to be f32
-            kernel_ctx.define_int(
-                    "PO_" + std::to_string(idx) + "_BIN_ARG_DT_IS_BF16", 0);
+            define_binary_po_type(data_type::f32);
         } else {
             memory_desc_t empty_mem_desc;
             dnnl_dims_t empty_dims = {1, 1, 1, 1};
@@ -1348,8 +1279,9 @@ inline status_t def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
             const memory_desc_wrapper src1_mdw(empty_mem_desc);
             const auto mdi = memory_desc_info_t::create(src1_mdw);
             def_memory_desc_info(kernel_ctx, mdi, bin_arg_name.c_str());
-            kernel_ctx.define_int(
-                    "PO_" + std::to_string(idx) + "_BIN_ARG_DT_IS_BF16", 0);
+
+            // unused - just need any type that's convertible to float
+            define_binary_po_type(data_type::f32);
         }
         if (e.is_eltwise(false)) {
             kernel_ctx.define_int(
@@ -1410,11 +1342,10 @@ inline status_t def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
     }
 
     kernel_ctx.define_int("POST_OP_CHAIN_LENGTH", nof_supported_post_ops);
-    if (post_ops.len() > 0) {
-        // due to C macro limitations on which post op service is build always
-        // load bf16 conversion functions
-        kernel_ctx.define_int("POST_OP_USING_BF16", 1);
-    }
+    if (post_op_uses_bf16) kernel_ctx.define_int("POST_OP_USING_BF16", 1);
+    if (post_op_uses_bf8) kernel_ctx.define_int("POST_OP_USING_BF8", 1);
+    if (post_op_uses_hf8) kernel_ctx.define_int("POST_OP_USING_HF8", 1);
+
     po_kernel_args += "\"";
     kernel_ctx.add_option(po_kernel_args);
     return status::success;
@@ -1512,6 +1443,7 @@ inline status_t def_attr_info_impl(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("WITH_SRC_SCALES", attr_info.with_src_scales);
     kernel_ctx.define_int("WITH_WEI_SCALES", attr_info.with_wei_scales);
     kernel_ctx.define_int("WITH_DST_SCALES", attr_info.with_dst_scales);
+    kernel_ctx.define_int("SRC_SCALES_MASK", attr_info.src_scales_mask);
     kernel_ctx.define_int("WEI_SCALES_MASK", attr_info.wei_scales_mask);
 
     kernel_ctx.define_int("WITH_SRC_ZPOINTS", attr_info.with_src_zpoints);
@@ -1521,6 +1453,10 @@ inline status_t def_attr_info_impl(compute::kernel_ctx_t &kernel_ctx,
             "WITH_SRC_ZPOINTS_PER_IC", attr_info.with_per_ic_src_zpoints);
     kernel_ctx.define_int(
             "WITH_DST_ZPOINTS_PER_OC", attr_info.with_per_oc_dst_zpoints);
+    kernel_ctx.define_int("WITH_WEI_ZPOINTS_DT_S8",
+            attr_info.wei_zpoints_data_type == dnnl_s8);
+    kernel_ctx.define_int("WITH_WEI_ZPOINTS_DT_U8",
+            attr_info.wei_zpoints_data_type == dnnl_u8);
 
     def_binary_alg_kinds(kernel_ctx);
     def_eltwise_alg_kinds(kernel_ctx);
