@@ -150,6 +150,13 @@ int fill_data(data_kind_t kind, const prb_t *prb, const cfg_t &cfg,
     int src_zp = kind == SRC && has_src_zp && src_zp_mask == 0 ? e_zp_src.value
                                                                : 0;
 
+    const auto &e_zp_wei = prb->attr.zero_points.get(DNNL_ARG_WEIGHTS);
+    const bool has_wei_zp = !e_zp_wei.is_def();
+    const int wei_zp_mask = attr_t::get_default_mask(e_zp_wei.policy);
+    // Apply wei_zp for weights tensor only.
+    int wei_zp = kind == WEI && has_wei_zp && wei_zp_mask == 0 ? e_zp_wei.value
+                                                               : 0;
+
     /* Do fixed partitioning to have same filling for any number of threads */
     const int64_t chunk_size = 64;
     const int64_t n_chunks = div_up(nelems, chunk_size);
@@ -176,7 +183,7 @@ int fill_data(data_kind_t kind, const prb_t *prb, const cfg_t &cfg,
             while (gen_val <= 0)
                 gen_val = gen(int_seed);
             float val = gen_val * (1.f + is_s8s8);
-            val += src_zp; // Add zp so that it will be subtracted.
+            val += src_zp + wei_zp; // Add zp so that it will be subtracted.
             mem_fp.set_elem(
                     0, round_to_nearest_representable(cfg.get_dt(kind), val));
             idx_start += 1;
@@ -186,7 +193,7 @@ int fill_data(data_kind_t kind, const prb_t *prb, const cfg_t &cfg,
             bool is_one = density == 1.f ? true : b_dist(b_seed);
             float gen_val = gen(int_seed) * (1.f + is_s8s8);
             float val = is_one * gen_val;
-            val += src_zp; // Add zp so that it will be subtracted.
+            val += src_zp + wei_zp; // Add zp so that it will be subtracted.
             mem_fp.set_elem(
                     idx, round_to_nearest_representable(cfg.get_dt(kind), val));
         }
@@ -206,16 +213,16 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
     bool force_f32_dt = init_pd_args.force_f32_dt;
 
     auto src_d = dnn_mem_t::init_md(prb->ndims, prb->src_dims().data(),
-            force_f32_dt ? dnnl_f32 : prb->get_dt(SRC),
-            normalize_tag(prb->stag, prb->ndims));
+            force_f32_dt ? dnnl_f32 : prb->get_dt(SRC), prb->stag,
+            prb->strides[STRIDES_SRC]);
     auto wei_d = dnn_mem_t::init_md(prb->ndims + prb->has_groups,
             prb->wei_dims().data(), force_f32_dt ? dnnl_f32 : prb->get_dt(WEI),
-            normalize_tag(prb->wtag, prb->ndims + prb->has_groups));
+            prb->wtag, prb->strides[STRIDES_WEI]);
     auto bia_d = dnn_mem_t::init_md(1, prb->bia_dims().data(),
             force_f32_dt ? dnnl_f32 : prb->get_dt(BIA), tag::any);
     auto dst_d = dnn_mem_t::init_md(prb->ndims, prb->dst_dims().data(),
-            force_f32_dt ? dnnl_f32 : prb->get_dt(DST),
-            normalize_tag(prb->dtag, prb->ndims));
+            force_f32_dt ? dnnl_f32 : prb->get_dt(DST), prb->dtag,
+            prb->strides[STRIDES_DST]);
 
     dnnl_alg_kind_t alg = dnnl_convolution_direct;
     if (prb->alg == WINO) alg = dnnl_convolution_winograd;
@@ -256,7 +263,7 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
                                               : dnnl_forward_training,
                             alg,
                             init_pd_args.src_md ? init_pd_args.src_md : src_d,
-                            wei_d, bia_d, dst_d, prb->strides().data(),
+                            wei_d, bia_d, dst_d, prb->kstrides().data(),
                             prb->dilations().data(), prb->padding().data(),
                             prb->padding_r().data(), dnnl_attr)));
             break;
@@ -264,7 +271,7 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
             TIME_C_PD(DNN_SAFE_STATUS(
                     dnnl_convolution_backward_data_primitive_desc_create(
                             &init_pd_args.pd, init_pd_args.engine, alg, src_d,
-                            wei_d, dst_d, prb->strides().data(),
+                            wei_d, dst_d, prb->kstrides().data(),
                             prb->dilations().data(), prb->padding().data(),
                             prb->padding_r().data(), init_pd_args.hint,
                             dnnl_attr)));
@@ -275,7 +282,7 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
             TIME_C_PD(DNN_SAFE_STATUS(
                     dnnl_convolution_backward_weights_primitive_desc_create(
                             &init_pd_args.pd, init_pd_args.engine, alg, src_d,
-                            wei_d, bia_d, dst_d, prb->strides().data(),
+                            wei_d, bia_d, dst_d, prb->kstrides().data(),
                             prb->dilations().data(), prb->padding().data(),
                             prb->padding_r().data(), init_pd_args.hint,
                             dnnl_attr)));
@@ -316,8 +323,8 @@ int init_prim_ref(benchdnn_dnnl_wrapper_t<dnnl_primitive_t> &prim_ref,
 
     for (const auto &prim_ref_dt_i : prim_ref_dt) {
         prb_t prb_cpu {*prb, prb->dir, prim_ref_dt_i, tag::any, tag::any,
-                tag::any, DIRECT, cpu_attr, prb->ctx_init, prb->ctx_exe,
-                prb->mb};
+                tag::any, {vdims_t(STRIDES_SIZE)}, DIRECT, cpu_attr,
+                prb->ctx_init, prb->ctx_exe, prb->mb};
 
         init_pd_args_t<prb_t> init_pd_args(
                 /* res = */ nullptr, get_cpu_engine(), &prb_cpu, prb->dir,
@@ -375,9 +382,12 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
                 = prb->get_dt(WEI) == dnnl_s8 || prb->get_dt(WEI) == dnnl_u8;
         const bool is_f16_dst = prb->get_dt(DST) == dnnl_f16;
         const bool is_x8x8f16 = is_int8_src && is_int8_wei && is_f16_dst;
+        const bool is_wei_zp = !prb->attr.zero_points.is_def(DNNL_ARG_WEIGHTS);
 
-        if (is_f32f32x8 || is_bf16bf16x8 || is_x8x8f16 || !is_valid_f16) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        if (is_f32f32x8 || is_bf16bf16x8 || is_x8x8f16 || !is_valid_f16
+                || is_wei_zp) {
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
@@ -386,13 +396,15 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
     // make sense to list all of them, just convert all unimplemented Winograd
     // problems into not supported.
     if (prb->alg == WINO) {
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 
     // GPU does not support depthwise fusion
     if (is_gpu() && prb->attr.post_ops.convolution_index() != -1) {
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 }
