@@ -99,7 +99,7 @@ static const std::map<int, std::vector<const char *>> supported_args {
         {DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS, {"attr_post_op_dw_wei"}},
 };
 
-static int str2arg(const std::string &str) {
+int str2arg(const std::string &str) {
     for (const auto &arg : supported_args)
         for (const auto &s : arg.second)
             if (str.compare(s) == 0) return arg.first;
@@ -118,7 +118,7 @@ static int str2arg(const std::string &str) {
     return DNNL_ARG_UNDEF;
 }
 
-static std::string arg2str(int arg) {
+std::string arg2str(int arg) {
     if (supported_args.find(arg) != supported_args.end())
         return std::string(supported_args.at(arg)[0]);
     if (arg & DNNL_ARG_MULTIPLE_SRC) {
@@ -164,6 +164,19 @@ const char *attr_t::policy2str(policy_t policy) {
     return "unknown attr_t::policy_t policy";
 }
 
+dnnl_rounding_mode_t str2rounding_mode(const std::string &str) {
+    std::string s(str);
+    // s.compare is lexicographical, case matters
+    std::transform(s.begin(), s.end(), s.begin(), ::tolower);
+#define CASE(_rm) \
+    if (s.compare(STRINGIFY(_rm)) == 0) return dnnl_rounding_mode_##_rm
+    CASE(environment);
+    CASE(stochastic);
+#undef CASE
+    assert(!"unknown attr_t::rounding_mode_t rounding_mode");
+    return dnnl_rounding_mode_environment;
+}
+
 int attr_t::get_default_mask(policy_t policy) {
     switch (policy) {
         case PER_DIM_0: return (1 << 0);
@@ -197,11 +210,13 @@ int attr_t::policy2mask(int arg, policy_t policy,
             default: SAFE(FAIL, CRIT); return -1;
         }
     } else if (prim_kind == dnnl_matmul) {
-        if (arg != DNNL_ARG_WEIGHTS || policy == policy_t::COMMON)
+        if ((arg != DNNL_ARG_SRC && arg != DNNL_ARG_WEIGHTS)
+                || policy == policy_t::COMMON)
             return attr_t::get_default_mask(policy);
 
-        if (ndims <= 0) SAFE_V(FAIL);
+        if (ndims < 2) SAFE_V(FAIL);
         switch (policy) {
+            case PER_DIM_1:
             case PER_OC: return (1 << (ndims - 1));
             case PER_OCIC: return (1 << (ndims - 1)) + (1 << (ndims - 2));
             default: SAFE_V(FAIL); return -1;
@@ -212,7 +227,7 @@ int attr_t::policy2mask(int arg, policy_t policy,
 
         // PER_OC
         assert(policy == policy_t::PER_OC);
-        if (ndims <= 0) SAFE_V(FAIL);
+        if (ndims < 1) SAFE_V(FAIL);
         return 1 << (ndims - 1);
     } else {
         // Default case
@@ -232,13 +247,13 @@ int parse_value_and_runtime(float &value, const std::string &s) {
                 "Error: scale or zero point input value is expected to be a "
                 "real number. Given input:",
                 s.c_str());
-        SAFE_V(FAIL);
+        SAFE(FAIL, WARN);
     }
     if (scale_pos != s.size()) {
         BENCHDNN_PRINT(0, "%s \'%s\'. %s \'%g\'.\n",
                 "Error: not every input symbol was processed. Given input:",
                 s.c_str(), "Parsed value:", value);
-        SAFE_V(FAIL);
+        SAFE(FAIL, WARN);
     }
     return OK;
 }
@@ -296,6 +311,7 @@ int attr_t::arg_scales_t::entry_t::from_str(const std::string &s) {
 
     if (!groups.empty()) {
         switch (this->policy) {
+            case PER_OC:
             case PER_OCIC:
                 if (this->groups.size() != 2) {
                     BENCHDNN_PRINT(0, "%s\n",
@@ -306,7 +322,8 @@ int attr_t::arg_scales_t::entry_t::from_str(const std::string &s) {
                 break;
             default:
                 BENCHDNN_PRINT(0, "%s\n",
-                        "Error: groups are supported only for policy PER_OCIC");
+                        "Error: groups are supported only for PER_OC and "
+                        "PER_OCIC policies.");
                 SAFE_V(FAIL);
         }
     }
@@ -585,7 +602,8 @@ bool attr_t::is_def(bool skip_fpmath) const {
             && scratchpad_mode == get_default_scratchpad_mode()
             && IMPLICATION(!skip_fpmath, fpmath_mode.is_def())
             && acc_mode == dnnl_accumulation_mode_strict
-            && deterministic.is_def();
+            && rounding_mode.is_def() && deterministic.is_def()
+            && dropout.is_def();
 }
 
 int attr_t::post_ops_t::find(pk_t kind, int start, int stop) const {
@@ -759,11 +777,28 @@ std::ostream &operator<<(std::ostream &s, dnnl_accumulation_mode_t am) {
     return s;
 }
 
+std::ostream &operator<<(std::ostream &s, const attr_t::rounding_mode_t &rm) {
+    std::string sep;
+    for (const auto &i : rm.rounding_modes_) {
+        s << sep << arg2str(i.first) << ":" << rounding_mode2str(i.second);
+        if (rm.is_set_seed) s << ":" << rm.seed;
+        sep = "+";
+    }
+    return s;
+}
+
 std::ostream &operator<<(std::ostream &s, const attr_t::deterministic_t &d) {
     s << bool2str(d.enabled);
     return s;
 }
 
+std::ostream &operator<<(std::ostream &s, const attr_t::dropout_t &drop) {
+    s << drop.p;
+    if ((drop.seed != 0) || (drop.tag != tag::any)) s << ":" << drop.seed;
+    if (drop.tag != tag::any) s << ":" << drop.tag;
+    return s;
+}
+
 std::ostream &operator<<(std::ostream &s, const attr_t &attr) {
     if (!attr.is_def()) {
         if (!attr.scales.is_def()) s << "--attr-scales=" << attr.scales << " ";
@@ -777,8 +812,12 @@ std::ostream &operator<<(std::ostream &s, const attr_t &attr) {
             s << "--attr-fpmath=" << attr.fpmath_mode << " ";
         if (attr.acc_mode != dnnl_accumulation_mode_strict)
             s << "--attr-acc-mode=" << attr.acc_mode << " ";
+        if (!attr.rounding_mode.is_def())
+            s << "--attr-rounding-mode=" << attr.rounding_mode << " ";
         if (!attr.deterministic.is_def())
             s << "--attr-deterministic=" << attr.deterministic << " ";
+        if (!attr.dropout.is_def())
+            s << "--attr-dropout=" << attr.dropout << " ";
     }
     return s;
 }
@@ -860,6 +899,8 @@ std::ostream &dump_global_params(std::ostream &s) {
         s << "--cpu-isa-hints=" << isa_hints_t::hints2str(hints) << " ";
     if (canonical || attr_same_pd_check != false)
         s << "--attr-same-pd-check=" << bool2str(attr_same_pd_check) << " ";
+    if (canonical || check_ref_impl != false)
+        s << "--check-ref-impl=" << bool2str(check_ref_impl) << " ";
 #if defined(DNNL_WITH_SYCL) || DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
     if (canonical || memory_kind != default_memory_kind)
         s << "--memory-kind=" << memory_kind << " ";
@@ -1013,6 +1054,13 @@ int attr_args_t::prepare_post_ops_mds(const attr_t &attr, int ndims,
         }
     }
 
+    // dropout
+    if (!attr.dropout.is_def()) {
+        auto drop_tensor_desc
+                = dnn_mem_t::init_md(ndims, dims, dnnl_u8, attr.dropout.tag);
+        mds.emplace(DNNL_ARG_ATTR_DROPOUT_MASK, std::move(drop_tensor_desc));
+    }
+
     return OK;
 }
 
@@ -1037,13 +1085,11 @@ dnnl_primitive_attr_t create_dnnl_attr(
             if (as.is_def(arg_name)) continue;
 
             const auto &e = arg.second;
-            // Weights mask differs from primitive to primitive, that's why it
-            // is stashed in `attr_args` at primitive creation time.
-            const bool is_wei_arg = arg_name == DNNL_ARG_WEIGHTS
-                    || arg_name
-                            == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
-            int mask = (is_wei_arg && e.policy != policy_t::COMMON)
-                    ? attr_args.get_mask(arg_name)
+            // Check if there's a arg with pre-defined mask in `attr_args`...
+            int args_mask = attr_args.get_mask(DNNL_ARG_ATTR_SCALES | arg_name);
+            // If it's non-default, use it, otherwise, deduce it.
+            int mask = args_mask != attr_args_t::undefined_mask
+                    ? args_mask
                     : attr_t::policy2mask(arg_name, e.policy);
 
             DNN_SAFE_V(dnnl_primitive_attr_set_scales(dnnl_attr, arg_name, mask,
@@ -1058,13 +1104,12 @@ dnnl_primitive_attr_t create_dnnl_attr(
             if (zp.is_def(arg_name)) continue;
 
             const auto &e = arg.second;
-            // Weights mask differs from primitive to primitive, that's why it
-            // is stashed in `attr_args` at primitive creation time.
-            const bool is_wei_arg = arg_name == DNNL_ARG_WEIGHTS
-                    || arg_name
-                            == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
-            int mask = (is_wei_arg && e.policy != policy_t::COMMON)
-                    ? attr_args.get_mask(DNNL_ARG_ATTR_ZERO_POINTS | arg_name)
+            // Check if there's a arg with pre-defined mask in `attr_args`...
+            int args_mask
+                    = attr_args.get_mask(DNNL_ARG_ATTR_ZERO_POINTS | arg_name);
+            // If it's non-default, use it, otherwise, deduce it.
+            int mask = args_mask != attr_args_t::undefined_mask
+                    ? args_mask
                     : attr_t::policy2mask(arg_name, e.policy);
 
             int ndims = static_cast<int>(e.groups.size());
@@ -1076,6 +1121,13 @@ dnnl_primitive_attr_t create_dnnl_attr(
         }
     }
 
+    if (!attr.rounding_mode.is_def()) {
+        for (const auto &e : attr.rounding_mode.rounding_modes_) {
+            DNN_SAFE_V(dnnl_primitive_attr_set_rounding(
+                    dnnl_attr, e.first, e.second));
+        }
+    }
+
     if (!attr.post_ops.is_def()) {
         dnnl_post_ops_t ops;
         DNN_SAFE_V(dnnl_post_ops_create(&ops));
@@ -1129,6 +1181,10 @@ dnnl_primitive_attr_t create_dnnl_attr(
     DNN_SAFE_V(dnnl_primitive_attr_set_deterministic(
             dnnl_attr, attr.deterministic.enabled));
 
+    if (!attr.dropout.is_def()) {
+        const auto &drop_mask_md = attr_args.get_md(DNNL_ARG_ATTR_DROPOUT_MASK);
+        DNN_SAFE_V(dnnl_primitive_attr_set_dropout(dnnl_attr, drop_mask_md));
+    }
     return dnnl_attr;
 }
 
@@ -1550,12 +1606,47 @@ float compute_binary(pk_t kind, float src0, float src1) {
     return NAN;
 }
 
+// This function is a full copy of ref_dropout(...) from the library.
+void maybe_dropout(const attr_t &attr, float &val, int64_t offset,
+        const dnn_mem_t &dropout_m) {
+
+    auto philox_bernoulli = [](float p, int seed, int64_t d) {
+        uint32_t r = dnnl::impl::math::philox4x32(d, seed);
+        p = std::max(std::min(p, 1.f), 0.f);
+        return (r > double(std::numeric_limits<uint32_t>::max()) * p);
+    };
+
+    if (!attr.dropout.is_def()) {
+        float p = attr.dropout.p;
+        int seed = attr.dropout.seed;
+        float inv_q = (p != 1.f) ? 1.f / (1.f - p) : 0.f;
+        uint8_t m = philox_bernoulli(p, seed, offset);
+        dropout_m.set_elem(offset, m);
+        val = (m) ? val * inv_q : 0;
+    }
+}
+
+void maybe_round(const attr_t &attr, int arg, float &val, int64_t offset,
+        dnnl_data_type_t dst_dt) {
+    uint32_t seed = attr.rounding_mode.seed;
+    switch (attr.rounding_mode.get(arg)) {
+        case dnnl_rounding_mode_stochastic:
+            val = dnnl::impl::math::stochastic_round_fwd(
+                    val, offset, seed, dst_dt);
+            break;
+        case dnnl_rounding_mode_environment: break;
+        default: assert(!"unknown rounding mode");
+    }
+}
+
 void maybe_post_ops(const attr_t &attr, float &val, float sum_val,
         const std::vector<float> &v_po_vals) {
+    const auto &po = attr.post_ops;
+    if (po.len() == 0) return;
+
     using namespace dnnl::impl::math;
 
     auto it_po = v_po_vals.begin();
-    const auto &po = attr.post_ops;
     for (int idx = 0; idx < po.len(); ++idx) {
         const auto &e = po.entry[idx];
 
