@@ -46,6 +46,7 @@
 #include "dnnl_memory.hpp"
 
 #include "utils/cold_cache.hpp"
+#include "utils/dnnl_query.hpp"
 #include "utils/fill.hpp"
 #include "utils/stream_kind.hpp"
 
@@ -187,6 +188,7 @@ int test_persistent_cache_api(
         std::vector<uint8_t> md_blob(sz);
         dnnl_memory_desc_get_blob(md_blob.data(), &sz, wei_md);
         dnnl_memory_desc_create_with_blob(&new_md, md_blob.data());
+        auto mew_mdw = make_benchdnn_dnnl_wrapper(new_md);
 
         if (dnnl_memory_desc_equal(wei_md, new_md) == 0)
             return res->state = FAILED, FAIL;
@@ -224,12 +226,15 @@ int test_persistent_cache_api(
         st = get_cache_blob(cache_blob, prim);
         if (st != OK) return res->state = FAILED, FAIL;
 
-        // The cross-engine reorder is a special primitive that may contain no
-        // kernels therefore the cache blob will always be empty, which is
-        // the correct behavior.
+        // The cross-engine and direct copy reorders are special primitives that
+        // may contain no kernels therefore the cache blob will always be empty,
+        // which is the correct behavior.
         if (cache_blob.empty()) {
             set_primitive_cache_capacity_without_clearing(old_capacity);
-            if (res->impl_name.find("cross_engine") != std::string::npos)
+            if (query_prim_kind(pd) == dnnl_reorder
+                    && (res->impl_name.find("cross_engine") != std::string::npos
+                            || res->impl_name.find("direct_copy")
+                                    != std::string::npos))
                 return OK;
 
             BENCHDNN_PRINT(
@@ -603,6 +608,8 @@ int measure_perf(
 std::vector<float> prepare_po_vals(const dnn_mem_t &dst_m, const args_t &args,
         const std::vector<std::pair<int, int>> &v_po_masks,
         const size_t dst_off) {
+    if (v_po_masks.empty()) return std::vector<float>();
+
     std::vector<float> v_vals(v_po_masks.size());
 
     for (size_t d = 0; d < v_po_masks.size(); ++d) {
@@ -635,16 +642,19 @@ void skip_unimplemented_data_type(
             || (is_cpu() && has_data_type_support(dnnl_f16)
                     && IMPLICATION(
                             !(dir & FLAG_INF), has_training_support(dnnl_f16)));
+    const bool has_e8m0_support
+            = is_gpu() || (is_cpu() && has_data_type_support(dnnl_e8m0));
     const bool has_f8_e5m2_support = is_gpu()
             || (is_cpu() && has_data_type_support(dnnl_f8_e5m2)
                     && (dir & FLAG_INF));
     const bool has_f8_e4m3_support = is_gpu()
             || (is_cpu() && has_data_type_support(dnnl_f8_e4m3)
-                    && (dir & FLAG_FWD));
+                    && (dir & FLAG_INF));
 #else
     const bool has_bf16_support = is_gpu();
     // f16 is supported on GPU for inference only.
     const bool has_f16_support = is_gpu() && (dir & FLAG_FWD);
+    const bool has_e8m0_support = is_gpu();
     const bool has_f8_e5m2_support = is_gpu();
     const bool has_f8_e4m3_support = is_gpu();
 #endif
@@ -655,12 +665,14 @@ void skip_unimplemented_data_type(
             case dnnl_bf16: need_skip = !has_bf16_support; break;
             case dnnl_f16: need_skip = !has_f16_support; break;
             case dnnl_f64: need_skip = !has_f64_support; break;
+            case dnnl_e8m0: need_skip = !has_e8m0_support; break;
             case dnnl_f8_e5m2: need_skip = !has_f8_e5m2_support; break;
             case dnnl_f8_e4m3: need_skip = !has_f8_e4m3_support; break;
             default: break;
         }
         if (need_skip) {
-            res->state = SKIPPED, res->reason = DATA_TYPE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::data_type_not_supported;
             return;
         }
     }
@@ -684,7 +696,8 @@ void skip_unimplemented_sum_po(const attr_t &attr, res_t *res,
             if (e.sum.zero_point != 0) {
                 // Sum with zero-point is only supported for int8
                 if (!is_integral_dt(src_dt)) {
-                    res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                    res->state = SKIPPED;
+                    res->reason = skip_reason::case_not_supported;
                     return;
                 } else {
                     // Only quantized sum operand can have zero point
@@ -692,7 +705,8 @@ void skip_unimplemented_sum_po(const attr_t &attr, res_t *res,
                             = e.sum.dt == dnnl_data_type_undef ? dst_dt
                                                                : e.sum.dt;
                     if (!is_integral_dt(e_sum_dt)) {
-                        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                        res->state = SKIPPED;
+                        res->reason = skip_reason::case_not_supported;
                         return;
                     }
                 }
@@ -700,19 +714,22 @@ void skip_unimplemented_sum_po(const attr_t &attr, res_t *res,
 
             // Sum with zero-point is not supported on GPU
             if (is_gpu() && e.sum.zero_point != 0) {
-                res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
                 break;
             }
             // Each sum must have same data on CPU
             if (is_cpu() && e.sum.dt != sum_dt) {
-                res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
                 break;
             }
             // Sum must have data type with the same size like dst on both
             if (dst_dt != dnnl_data_type_undef && sum_dt != dnnl_data_type_undef
                     && dnnl_data_type_size(dst_dt)
                             != dnnl_data_type_size(e.sum.dt)) {
-                res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+                res->state = SKIPPED;
+                res->reason = skip_reason::case_not_supported;
                 return;
             }
         }
@@ -732,14 +749,18 @@ void skip_unimplemented_prelu_po(
         case dnnl_deconvolution:
         case dnnl_inner_product:
         case dnnl_matmul: return; break;
-        default: res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED; break;
+        default:
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
+            break;
     }
 }
 
 void skip_unimplemented_arg_scale(const attr_t &attr, res_t *res) {
     for (const auto &arg_s : attr.scales.scales) {
         if (arg_s.second.policy != policy_t::COMMON) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+            res->state = SKIPPED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
@@ -756,13 +777,15 @@ void skip_invalid_inplace(res_t *res, dnnl_data_type_t sdt,
     // General limitation of in-place mode is having same amount of memory on
     // input and output.
     if (sdt != ddt) {
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
         return;
     }
 
     if (dtag == tag::any) return;
     if (stag != dtag) {
-        res->state = SKIPPED, res->reason = INVALID_CASE;
+        res->state = SKIPPED;
+        res->reason = skip_reason::invalid_case;
         return;
     }
 }
@@ -779,6 +802,19 @@ int check_same_pd(const dnnl_primitive_desc_t &pd_no_attr, res_t *res) {
     return FAIL;
 }
 
+// Checks if unexpected reference implementation was hit.
+int check_ref_impl_hit(res_t *res) {
+    if (!check_ref_impl) return OK;
+
+    const auto &impl_name = res->impl_name;
+    if (impl_name.find("ref") != std::string::npos) {
+        res->state = FAILED;
+        res->reason = "Ref Impl Not Expected";
+        return FAIL;
+    }
+    return OK;
+}
+
 bool is_cpu(const dnnl_engine_t &engine) {
     return query_engine_kind(engine) == dnnl_cpu;
 }
@@ -861,7 +897,7 @@ bool is_f64_supported(const dnnl_engine_t &engine) {
 #if defined(_WIN32)
 #include "windows.h"
 
-static size_t get_cpu_ram_size() {
+size_t get_cpu_ram_size() {
     MEMORYSTATUSEX s {};
     s.dwLength = sizeof(s);
     GlobalMemoryStatusEx(&s);
@@ -871,7 +907,7 @@ static size_t get_cpu_ram_size() {
 #include <unistd.h>
 #include <sys/sysctl.h>
 
-static size_t get_cpu_ram_size() {
+size_t get_cpu_ram_size() {
 #ifdef __APPLE__
     int query_ram[] = {CTL_HW, HW_MEMSIZE};
 #else
@@ -887,14 +923,14 @@ static size_t get_cpu_ram_size() {
 #else
 #include <sys/sysinfo.h>
 
-static size_t get_cpu_ram_size() {
+size_t get_cpu_ram_size() {
     struct sysinfo s {};
     sysinfo(&s);
     return s.totalram;
 }
 #endif
 
-static int get_gpu_ram_sizes(size_t &ram_size, size_t &max_alloc_size) {
+int get_gpu_ram_sizes(size_t &ram_size, size_t &max_alloc_size) {
     if (!is_gpu()) return OK;
     if (ram_size > 0 && max_alloc_size > 0) return OK;
 
@@ -977,35 +1013,6 @@ int get_gpu_cache_size(size_t &cache_size) {
     return OK;
 }
 
-struct check_mem_size_args_t {
-    check_mem_size_args_t(const_dnnl_primitive_desc_t pd, bool want_input)
-        : pd(pd)
-        , want_input(want_input)
-        , is_scratchpad(false)
-        , total_size_device(0)
-        , total_size_cpu(0)
-        , scratchpad_size(0) {}
-
-    // Input args.
-    const_dnnl_primitive_desc_t pd;
-    bool want_input;
-    bool is_scratchpad;
-
-    // Output args:
-    // `sizes` used to validate OpenCL memory requirements.
-    std::vector<size_t> sizes;
-    // `total_size_device` specifies memory allocated on device for a test obj.
-    size_t total_size_device;
-    // `total_size_cpu` specifies:
-    // * Memory allocated for reference ocmputations (`C` mode only).
-    // * Memory allocated for comparison results (`C` mode only).
-    // * Memory allocated for mapping device memory (GPU backend only).
-    // * Memory allocated on CPU for a test obj (CPU backend only).
-    size_t total_size_cpu;
-    // `scratchpad_size` specifies a scratchpad size for specific checks.
-    size_t scratchpad_size;
-};
-
 static int check_total_size(
         const check_mem_size_args_t &check_mem_size_args, res_t *res) {
     static size_t cpu_device_capacity = get_cpu_ram_size();
@@ -1036,7 +1043,7 @@ static int check_total_size(
                     "[CHECK_MEM][%s]: Not enough device RAM for a problem.\n",
                     dir_c_str());
             res->state = SKIPPED;
-            res->reason = NOT_ENOUGH_RAM;
+            res->reason = skip_reason::not_enough_ram;
         }
 
         const bool all_allocation_fit_limit = std::all_of(
@@ -1053,7 +1060,7 @@ static int check_total_size(
                 });
         if (!all_allocation_fit_limit) {
             res->state = SKIPPED;
-            res->reason = NOT_ENOUGH_RAM;
+            res->reason = skip_reason::not_enough_ram;
         }
 
         BENCHDNN_PRINT((!fits_device_ram ? 2 : 6),
@@ -1091,7 +1098,7 @@ static int check_total_size(
         } else {
             res->state = SKIPPED;
         }
-        res->reason = NOT_ENOUGH_RAM;
+        res->reason = skip_reason::not_enough_ram;
     }
 
     BENCHDNN_PRINT((!fits_cpu_ram ? 2 : 6),
@@ -1138,6 +1145,10 @@ void add_md_size(const_dnnl_memory_desc_t md,
         auto ref_md = dnn_mem_t::init_md(
                 query_md_ndims(md), query_md_dims(md), dnnl_f32, tag::abx);
         const auto ref_md_size = dnnl_memory_desc_get_size(ref_md);
+
+        const size_t ref_mem_idx = check_mem_size_args.want_input ? 0 : 1;
+        check_mem_size_args.total_ref_md_size[ref_mem_idx] = ref_md_size;
+
         // A memory copy for ref_compute, happens only in correctness.
         check_mem_size_args.total_size_cpu += is_corr * ref_md_size;
 
@@ -1156,13 +1167,18 @@ void add_md_size(const_dnnl_memory_desc_t md,
     }
 }
 
+bool is_fwd_training(dnnl_prop_kind_t prop_kind) {
+    return prop_kind == dnnl_forward_training
+            || prop_kind == dnnl_prop_kind_undef;
+}
+
 bool is_fwd_prop_kind(dnnl_prop_kind_t prop_kind) {
     return prop_kind == dnnl_forward_training
             || prop_kind == dnnl_forward_inference
             || prop_kind == dnnl_prop_kind_undef;
 }
 
-static void get_memory_bytes(check_mem_size_args_t &check_mem_size_args) {
+void get_memory_bytes(check_mem_size_args_t &check_mem_size_args) {
     auto const_pd = check_mem_size_args.pd;
     const int n_idx = check_mem_size_args.want_input
             ? query_n_inputs(const_pd)
@@ -1218,8 +1234,8 @@ int check_mem_size(const_dnnl_memory_desc_t md, res_t *res) {
     return check_total_size(check_mem_size_args, res);
 }
 
-int check_mem_size(
-        const_dnnl_primitive_desc_t const_pd, res_t *res, dir_t dir) {
+int check_mem_size(const_dnnl_primitive_desc_t const_pd, res_t *res, dir_t dir,
+        bool need_skip) {
     // Skip the check if it is disabled.
     if (!mem_check) return OK;
 
@@ -1230,7 +1246,7 @@ int check_mem_size(
     // repreated run when the second test object is created to test the
     // primitive cache, but allows to verify both objects when a double-run
     // driver executes fwd-for-bwd first and bwd after.
-    if (res->mem_check_dir == dir) return OK;
+    if (need_skip && res->mem_check_dir == dir) return OK;
     res->mem_check_dir = dir;
 
     // Get input sizes.
@@ -1257,6 +1273,9 @@ int check_mem_size(
     check_mem_size_args.want_input = false;
     get_memory_bytes(check_mem_size_args);
 
+    // Save the mem size args for graph driver check.
+    res->mem_size_args = check_mem_size_args;
+
     return check_total_size(check_mem_size_args, res);
 }
 
@@ -1268,7 +1287,8 @@ int get_memory_footprint(const_dnnl_primitive_desc_t const_pd, res_t *res) {
             const_pd, /* want_input = */ false);
     get_memory_bytes(check_mem_out_size_args); // Get output bytes.
 
-    // Update read bytes with dst bytes in case of sum post-op.
+    // Sum post-ops include dst bytes as an input. Not included in get_memory_bytes
+    // since it would cause check_mem_size to double-count dst bytes.
     auto const_attr_po = query_post_ops(const_pd);
     auto po_len = dnnl_post_ops_len(const_attr_po);
     for (int idx = 0; idx < po_len; ++idx) {
@@ -1276,13 +1296,38 @@ int get_memory_footprint(const_dnnl_primitive_desc_t const_pd, res_t *res) {
         if (kind == dnnl_sum) {
             const auto &dst_md = query_md(const_pd, DNNL_ARG_DST);
             add_md_size(dst_md, check_mem_in_size_args);
-        } else if (kind == dnnl_binary) {
-            int po_arg = DNNL_ARG_ATTR_MULTIPLE_POST_OP(idx) | DNNL_ARG_SRC_1;
-            const auto &po_md = query_md(const_pd, po_arg);
-            add_md_size(po_md, check_mem_in_size_args);
         }
     }
 
+    // Forward nearest neighbor down sampling (one or more bigger source
+    // dimensions than in destination) cases by the nature of the operation
+    // read only a part of a source tensor. Here we adjust the `ibytes` variable
+    // so that reported bandwidth numbers reflect the reality, otherwise
+    // final numbers are much higher than they should be.
+    dnnl_primitive_kind_t kind = query_prim_kind(const_pd);
+    dnnl_alg_kind_t alg = query_alg_kind(const_pd);
+    dnnl_prop_kind_t prop = query_prop_kind(const_pd);
+    if (is_fwd_prop_kind(prop) && kind == dnnl_resampling
+            && alg == dnnl_resampling_nearest) {
+        auto src_md = query_md(const_pd, DNNL_ARG_SRC);
+        auto dst_md = query_md(const_pd, DNNL_ARG_DST);
+        auto ndims = query_md_ndims(src_md);
+        auto src_pdims = query_md_padded_dims(src_md);
+        auto dst_pdims = query_md_padded_dims(dst_md);
+        dnnl_dim_t total_elems = 1;
+        dnnl_dim_t read_elems = 1;
+        for (int i = 0; i < ndims; i++) {
+            if (dst_pdims[i] < src_pdims[i]) {
+                total_elems *= src_pdims[i];
+                read_elems *= dst_pdims[i];
+            }
+        }
+        const auto src_size = dnnl_memory_desc_get_size(src_md);
+        const auto fixed_src_size = src_size / static_cast<size_t>(total_elems)
+                * static_cast<size_t>(read_elems);
+        check_mem_in_size_args.total_size_device += fixed_src_size - src_size;
+    }
+
     res->ibytes = check_mem_in_size_args.total_size_device;
     res->obytes = check_mem_out_size_args.total_size_device;
 
@@ -1425,18 +1470,32 @@ float reorder_rescale_factor() {
     return factor;
 }
 
-dims_t md2dims(const_dnnl_memory_desc_t md, int mask, bool extend_by_ones) {
+dims_t md2dims(const_dnnl_memory_desc_t md, int mask, bool extend_by_ones,
+        const std::vector<int64_t> &groups) {
     auto ndims = query_md_ndims(md);
     dims_t dims;
     for (int d = 0; d < ndims; ++d) {
-        if (mask & (1 << d))
+        if (mask & (1 << d)) {
             dims.push_back(query_md_dims(md)[d]);
-        else if (extend_by_ones)
+            // Note: groups are done for matmul's last two dimensions.
+            const auto group_dim = d - (ndims - 2);
+            if (!groups.empty() && group_dim >= 0) {
+                // If groups are passed, divide dims on the correspondent group
+                // size. It's needed to pass proper memory objects.
+                assert(dims.back() % groups[group_dim] == 0);
+                dims.back() /= groups[group_dim];
+            }
+        } else if (extend_by_ones) {
             dims.push_back(1);
+        }
     }
     return dims;
 }
 
+dims_t md2dims(const_dnnl_memory_desc_t md, int mask, bool extend_by_ones) {
+    return md2dims(md, mask, extend_by_ones, {});
+}
+
 dnnl_data_type_t deduce_cfg_data_type(
         dnnl_data_type_t in_dt, const attr_t &attr, data_kind_t dk) {
     dnnl_data_type_t dt_ = in_dt;
@@ -1446,6 +1505,7 @@ dnnl_data_type_t deduce_cfg_data_type(
         switch (attr.fpmath_mode.mode) {
             case dnnl_fpmath_mode_strict: break;
             case dnnl_fpmath_mode_bf16: dt_ = dnnl_bf16; break;
+            case dnnl_fpmath_mode_f16: dt_ = dnnl_f16; break;
             case dnnl_fpmath_mode_tf32: dt_ = dnnl_bf16; break;
             default: assert(!"unsupported_fpmath_mode"); SAFE_V(CRIT);
         }
@@ -1558,6 +1618,9 @@ int init_ref_memory_args_default_case(int exec_arg, dnn_mem_t &mem,
     const bool is_post_ops_arg = (exec_arg & post_ops_range);
     const bool is_scales_arg = (exec_arg & DNNL_ARG_ATTR_SCALES);
     const bool is_zero_point_arg = (exec_arg & DNNL_ARG_ATTR_ZERO_POINTS);
+    const bool is_dropout_p = (exec_arg == DNNL_ARG_ATTR_DROPOUT_PROBABILITY);
+    const bool is_dropout_seed = (exec_arg == DNNL_ARG_ATTR_DROPOUT_SEED);
+    const bool is_rounding_seed = (exec_arg == DNNL_ARG_ATTR_ROUNDING_SEED);
 
     if (is_post_ops_arg) {
         if (exec_arg & DNNL_ARG_SRC_1) {
@@ -1592,6 +1655,15 @@ int init_ref_memory_args_default_case(int exec_arg, dnn_mem_t &mem,
         int local_exec_arg = exec_arg ^ DNNL_ARG_ATTR_ZERO_POINTS;
         TIME_FILL(SAFE(
                 fill_zero_points(attr, local_exec_arg, mem, ref_mem), WARN));
+    } else if (is_dropout_p) {
+        ref_mem.set_elem(0, attr.dropout.p);
+        TIME_FILL(SAFE(mem.reorder(ref_mem), WARN));
+    } else if (is_dropout_seed) {
+        ref_mem.set_elem(0, attr.dropout.seed);
+        TIME_FILL(SAFE(mem.reorder(ref_mem), WARN));
+    } else if (is_rounding_seed) {
+        ref_mem.set_elem(0, attr.rounding_mode.seed);
+        TIME_FILL(SAFE(mem.reorder(ref_mem), WARN));
     }
 
     return OK;
@@ -1629,7 +1701,12 @@ int check_bitwise(dnnl_primitive_t prim, const std::vector<data_kind_t> &kinds,
         assert(arg > 0);
 
         auto &mem = args.find(arg);
-        SAFE_V(bool(mem) ? OK : FAIL);
+        if (!mem) {
+            BENCHDNN_PRINT(0, "%s\n",
+                    "Error: output memory was not found among arguments.");
+            res->state = FAILED;
+            return FAIL;
+        }
         // A memory used as reference for comparison, must be allocated on the
         // CPU engine.
         run1_mem_map.emplace(
@@ -1654,9 +1731,20 @@ int check_bitwise(dnnl_primitive_t prim, const std::vector<data_kind_t> &kinds,
                 ? (has_multiple_args ? DNNL_ARG_MULTIPLE_SRC : DNNL_ARG_SRC)
                 : DNNL_ARG_DIFF_DST;
         auto &in_mem = const_cast<dnn_mem_t &>(args.find(query_arg));
-        SAFE_V(bool(in_mem) ? OK : FAIL);
+        if (!in_mem) {
+            BENCHDNN_PRINT(0, "%s\n",
+                    "Error: input memory was not found among arguments.");
+            res->state = FAILED;
+            return FAIL;
+        }
         const auto &orig_in_mem = args.find(-query_arg);
-        SAFE_V(bool(orig_in_mem) ? OK : FAIL);
+        if (!orig_in_mem) {
+            BENCHDNN_PRINT(0, "%s\n",
+                    "Error: original input memory was not found among "
+                    "arguments.");
+            res->state = FAILED;
+            return FAIL;
+        }
         SAFE(in_mem.reorder(orig_in_mem), WARN);
     }
 
