@@ -29,7 +29,7 @@
 
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
 #include "oneapi/dnnl/dnnl_ocl.hpp"
-#include "src/gpu/intel/ocl/ocl_usm_utils.hpp"
+#include "src/xpu/ocl/usm_utils.hpp"
 #endif
 
 #include "tests/test_thread.hpp"
@@ -206,6 +206,9 @@ float dnn_mem_t::get_elem(int64_t idx, int buffer_index) const {
         case dnnl_bf16:
             elem = static_cast<dnnl::impl::bfloat16_t *>(data)[idx];
             break;
+        case dnnl_e8m0:
+            elem = static_cast<dnnl::impl::float8_e8m0_t *>(data)[idx];
+            break;
         case dnnl_f8_e5m2:
             elem = static_cast<dnnl::impl::float8_e5m2_t *>(data)[idx];
             break;
@@ -242,6 +245,7 @@ void dnn_mem_t::set_elem(int64_t idx, float value, int buffer_index) const {
         case dnnl_f64: ((double *)data)[idx] = value; break;
         case dnnl_f16: ((dnnl::impl::float16_t *)data)[idx] = value; break;
         case dnnl_bf16: ((dnnl::impl::bfloat16_t *)data)[idx] = value; break;
+        case dnnl_e8m0: ((dnnl::impl::float8_e8m0_t *)data)[idx] = value; break;
         case dnnl_f8_e5m2:
             ((dnnl::impl::float8_e5m2_t *)data)[idx] = value;
             break;
@@ -273,8 +277,6 @@ void dnn_mem_t::set_elem(int64_t idx, float value, int buffer_index) const {
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL || defined(DNNL_WITH_SYCL)
 static int init_memory(
         dnnl_memory_t *ret, const dnnl_memory_desc_t &md, dnnl_memory_t mem) {
-    void *handle;
-    DNN_SAFE(dnnl_memory_get_data_handle(mem, &handle), CRIT);
 
     dnnl_engine_t engine;
     DNN_SAFE(dnnl_memory_get_engine(mem, &engine), CRIT);
@@ -284,22 +286,43 @@ static int init_memory(
 
     *ret = nullptr;
 
+    const int nhandles = query_md_num_handles(md);
+    std::vector<void *> handles(nhandles);
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+    for (int i = 0; i < nhandles; i++)
+        DNN_SAFE(dnnl_memory_get_data_handle_v2(mem, &handles[i], i), CRIT);
+#else
+    DNN_SAFE(dnnl_memory_get_data_handle(mem, &handles[0]), CRIT);
+#endif
+
     if (is_opencl) {
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
         dnnl_ocl_interop_memory_kind_t mem_kind;
         DNN_SAFE(dnnl_ocl_interop_memory_get_memory_kind(mem, &mem_kind), CRIT);
-        DNN_SAFE(dnnl_ocl_interop_memory_create(
-                         ret, md, engine, mem_kind, handle),
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+        DNN_SAFE(dnnl_ocl_interop_memory_create_v2(ret, md, engine, mem_kind,
+                         (int)handles.size(), handles.data()),
                 CRIT);
+#else
+        DNN_SAFE(dnnl_ocl_interop_memory_create(
+                         ret, md, engine, mem_kind, handles[0]),
+                CRIT);
+#endif
 #endif
     } else if (is_sycl) {
 #ifdef DNNL_WITH_SYCL
         dnnl_sycl_interop_memory_kind_t mem_kind;
         DNN_SAFE(
                 dnnl_sycl_interop_memory_get_memory_kind(mem, &mem_kind), CRIT);
-        DNN_SAFE(dnnl_sycl_interop_memory_create(
-                         ret, md, engine, mem_kind, handle),
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+        DNN_SAFE(dnnl_sycl_interop_memory_create_v2(ret, md, engine, mem_kind,
+                         (int)handles.size(), handles.data()),
                 CRIT);
+#else
+        DNN_SAFE(dnnl_sycl_interop_memory_create(
+                         ret, md, engine, mem_kind, handles[0]),
+                CRIT);
+#endif
 #endif
     }
 
@@ -378,7 +401,7 @@ void dnn_mem_t::memset(int value, size_t size) const {
             case memory_kind_ext_t::usm:
             case memory_kind_ext_t::usm_device:
             case memory_kind_ext_t::usm_shared: {
-                DNN_SAFE_V(dnnl::impl::gpu::intel::ocl::usm::memset(
+                DNN_SAFE_V(dnnl::impl::xpu::ocl::usm::memset(
                         stream, mem_handle, value, size));
                 DNN_SAFE_V(dnnl_stream_wait(stream));
                 return;
@@ -449,6 +472,10 @@ size_t dnn_mem_t::pad_memory_size(
 dnnl_memory_desc_t dnn_mem_t::pad_memory_desc(const_dnnl_memory_desc_t md,
         dnnl_engine_kind_t engine_kind, bool *was_padded) {
     if (was_padded) *was_padded = false;
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+    // TODO: add padded memory descriptor support for sparse memory.
+    if (query_md_format_kind(md) == dnnl_format_kind_sparse) return nullptr;
+#endif
     size_t old_sz = dnnl_memory_desc_get_size(md);
     if (old_sz == 0 || !has_bench_mode_bit(mode_bit_t::corr)
             || engine_kind == dnnl_cpu)
@@ -503,6 +530,15 @@ benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_csr_md(int ndims,
     return md;
 }
 
+benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_coo_md(int ndims,
+        const dnnl_dims_t dims, dnnl_data_type_t data_type, dnnl_dim_t nnz,
+        dnnl_data_type_t indices_dt) {
+    dnnl_memory_desc_t md {};
+    DNN_SAFE_V(dnnl_memory_desc_create_with_coo_encoding(
+            &md, ndims, dims, data_type, nnz, indices_dt));
+    return md;
+}
+
 benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> dnn_mem_t::init_sparse_packed_md(
         int ndims, const dnnl_dims_t dims, dnnl_data_type_t data_type,
         dnnl_dim_t nnz) {
@@ -517,9 +553,18 @@ int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
 #ifdef DNNL_WITH_SYCL
     if (handle_info.is_host_ptr) {
         // Ignore memory_kind with host pointers and force USM.
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+        const int nhandles = query_md_num_handles(md_);
+        std::vector<void *> handles(nhandles, handle_info.ptr);
+        DNN_SAFE(dnnl_sycl_interop_memory_create_v2(&m_, md_, engine_,
+                         dnnl_sycl_interop_usm, (int)handles.size(),
+                         handles.data()),
+                CRIT);
+#else
         DNN_SAFE(dnnl_sycl_interop_memory_create(&m_, md_, engine_,
                          dnnl_sycl_interop_usm, handle_info.ptr),
                 CRIT);
+#endif
         return OK;
     }
 
@@ -533,9 +578,18 @@ int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
                     = (memory_kind == memory_kind_ext_t::usm
                                     ? dnnl_sycl_interop_usm
                                     : dnnl_sycl_interop_buffer);
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+            const int nhandles = query_md_num_handles(md_);
+            std::vector<void *> handles(nhandles, handle_info.ptr);
+            DNN_SAFE(dnnl_sycl_interop_memory_create_v2(&m_padded_, md_padded,
+                             engine_, mem_kind, (int)handles.size(),
+                             handles.data()),
+                    CRIT);
+#else
             DNN_SAFE(dnnl_sycl_interop_memory_create(&m_padded_, md_padded,
                              engine_, mem_kind, handle_info.ptr),
                     CRIT);
+#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -543,21 +597,39 @@ int dnn_mem_t::initialize_memory_create_sycl(const handle_info_t &handle_info) {
         case memory_kind_ext_t::usm_shared: {
             SAFE(handle_info.is_allocate() ? OK : FAIL, CRIT);
             is_data_owner_ = true;
-            size_t sz = dnnl_memory_desc_get_size(md_padded);
+
             auto eng = dnnl::engine(engine_, true);
             auto dev = dnnl::sycl_interop::get_device(eng);
             auto ctx = dnnl::sycl_interop::get_context(eng);
-            if (memory_kind == memory_kind_ext_t::usm_device) {
-                data_.push_back(::sycl::malloc_device(sz, dev, ctx));
-            } else {
-                data_.push_back(::sycl::malloc_shared(sz, dev, ctx));
+
+            const int nhandles = query_md_num_handles(md_);
+            for (int i = 0; i < nhandles; i++) {
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+                size_t sz = dnnl_memory_desc_get_size_v2(md_padded, i);
+#else
+                size_t sz = dnnl_memory_desc_get_size(md_padded);
+#endif
+                if (memory_kind == memory_kind_ext_t::usm_device) {
+                    data_.push_back(::sycl::malloc_device(sz, dev, ctx));
+                } else {
+                    data_.push_back(::sycl::malloc_shared(sz, dev, ctx));
+                }
+                if (sz > 0 && !data_[i]) {
+                    for (void *p : data_)
+                        ::sycl::free(p, ctx);
+                    DNN_SAFE(dnnl_out_of_memory, CRIT);
+                }
             }
-            assert(data_.size() == 1);
-            DNN_SAFE((sz > 0 && !data_[0]) ? dnnl_out_of_memory : dnnl_success,
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+            DNN_SAFE(dnnl_sycl_interop_memory_create_v2(&m_padded_, md_padded,
+                             engine_, dnnl_sycl_interop_usm, (int)data_.size(),
+                             data_.data()),
                     CRIT);
+#else
             DNN_SAFE(dnnl_sycl_interop_memory_create(&m_padded_, md_padded,
                              engine_, dnnl_sycl_interop_usm, data_[0]),
                     CRIT);
+#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -576,9 +648,18 @@ int dnn_mem_t::initialize_memory_create_opencl(
 #if DNNL_GPU_RUNTIME == DNNL_RUNTIME_OCL
     if (handle_info.is_host_ptr) {
         // Ignore memory_kind with host pointers and force USM.
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+        const int nhandles = query_md_num_handles(md_);
+        std::vector<void *> handles(nhandles, handle_info.ptr);
+        DNN_SAFE(dnnl_ocl_interop_memory_create_v2(&m_, md_, engine_,
+                         dnnl_ocl_interop_usm, (int)handles.size(),
+                         handles.data()),
+                CRIT);
+#else
         DNN_SAFE(dnnl_ocl_interop_memory_create(&m_, md_, engine_,
                          dnnl_ocl_interop_usm, handle_info.ptr),
                 CRIT);
+#endif
         return OK;
     }
 
@@ -594,29 +675,56 @@ int dnn_mem_t::initialize_memory_create_opencl(
                     = (memory_kind == memory_kind_ext_t::usm
                                     ? dnnl_ocl_interop_usm
                                     : dnnl_ocl_interop_buffer);
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+            const int nhandles = query_md_num_handles(md_);
+            std::vector<void *> handles(nhandles, handle_info.ptr);
+            DNN_SAFE(dnnl_ocl_interop_memory_create_v2(&m_padded_, md_padded,
+                             engine_, mem_kind, (int)handles.size(),
+                             handles.data()),
+                    CRIT);
+#else
             DNN_SAFE(dnnl_ocl_interop_memory_create(&m_padded_, md_padded,
                              engine_, mem_kind, handle_info.ptr),
                     CRIT);
+#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
         case memory_kind_ext_t::usm_device:
         case memory_kind_ext_t::usm_shared: {
             is_data_owner_ = true;
-            size_t sz = dnnl_memory_desc_get_size(md_padded);
-            if (memory_kind == memory_kind_ext_t::usm_device) {
-                data_.push_back(dnnl::impl::gpu::intel::ocl::usm::malloc_device(
-                        engine_, sz));
-            } else {
-                data_.push_back(dnnl::impl::gpu::intel::ocl::usm::malloc_shared(
-                        engine_, sz));
+
+            const int nhandles = query_md_num_handles(md_);
+            for (int i = 0; i < nhandles; i++) {
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+                size_t sz = dnnl_memory_desc_get_size_v2(md_padded, i);
+#else
+                size_t sz = dnnl_memory_desc_get_size(md_padded);
+#endif
+                if (memory_kind == memory_kind_ext_t::usm_device) {
+                    data_.push_back(dnnl::impl::xpu::ocl::usm::malloc_device(
+                            engine_, sz));
+                } else {
+                    data_.push_back(dnnl::impl::xpu::ocl::usm::malloc_shared(
+                            engine_, sz));
+                }
+
+                if (sz > 0 && !data_[i]) {
+                    for (void *p : data_)
+                        dnnl::impl::xpu::ocl::usm::free(engine_, p);
+                    DNN_SAFE(dnnl_out_of_memory, CRIT);
+                }
             }
-            assert(data_.size() == 1);
-            DNN_SAFE((sz > 0 && !data_[0]) ? dnnl_out_of_memory : dnnl_success,
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+            DNN_SAFE(dnnl_ocl_interop_memory_create_v2(&m_padded_, md_padded,
+                             engine_, dnnl_ocl_interop_usm, (int)data_.size(),
+                             data_.data()),
                     CRIT);
+#else
             DNN_SAFE(dnnl_ocl_interop_memory_create(&m_padded_, md_padded,
                              engine_, dnnl_ocl_interop_usm, data_[0]),
                     CRIT);
+#endif
             SAFE(init_memory(&m_, md_, m_padded_), CRIT);
             break;
         }
@@ -752,7 +860,7 @@ static int cleanup_opencl(
         case memory_kind_ext_t::usm_device:
         case memory_kind_ext_t::usm_shared:
             for (void *p : data)
-                dnnl::impl::gpu::intel::ocl::usm::free(engine, p);
+                dnnl::impl::xpu::ocl::usm::free(engine, p);
             break;
         default: break;
     }
@@ -926,6 +1034,7 @@ int check_zero_padding(
         case dnnl_data_type_undef:
             return OK;
 
+            CASE(dnnl_e8m0, dnnl::impl::float8_e8m0_t);
             CASE(dnnl_f8_e5m2, dnnl::impl::float8_e5m2_t);
             CASE(dnnl_f8_e4m3, dnnl::impl::float8_e4m3_t);
             CASE(dnnl_bf16, dnnl::impl::bfloat16_t);
