@@ -21,6 +21,7 @@
 #include <vector>
 #include <unordered_map>
 
+#include "allocator.hpp"
 #include "dnnl_common.hpp"
 #include "graph.hpp"
 #include "ref_partition.hpp"
@@ -341,7 +342,8 @@ using namespace dnnl::graph;
 std::string case_to_str(const std::string &json_file,
         const std::map<size_t, std::string> &in_shapes,
         const std::map<size_t, std::string> &op_attrs,
-        const std::string &fpmath_mode, const int64_t mb) {
+        const std::string &fpmath_mode, const size_t expected_n_partitions,
+        const int64_t mb) {
     std::stringstream s;
     dump_global_params(s);
 
@@ -374,6 +376,11 @@ std::string case_to_str(const std::string &json_file,
         s << "--attr-fpmath=" << fpmath_mode << " ";
     }
 
+    if (expected_n_partitions != 0) {
+        s << "--expected-n-partitions=" << std::to_string(expected_n_partitions)
+          << " ";
+    }
+
     s << "--case=" << json_file;
     return s.str();
 }
@@ -385,13 +392,12 @@ void skip_unimplemented_ops(const dnnl::graph::partition &partition,
     // TODO: extend with `getenv` call if limits too much.
     if (is_gc_backend()) {
         res->state = SKIPPED;
-        res->reason = CASE_NOT_SUPPORTED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 
     // A list of ops that don't have DNNL backend support so far.
-    static const std::vector<std::string> unimplemented_ops {
-            "Pow", "StaticReshape", "StaticTranspose"};
+    static const std::vector<std::string> unimplemented_ops {"Pow"};
 
     // For an unsupported partition, retrieve all operation IDs, find a
     // correspondent operation kind in a deserialized_graph and match it against
@@ -408,7 +414,7 @@ void skip_unimplemented_ops(const dnnl::graph::partition &partition,
             BENCHDNN_PRINT(
                     2, "[INFO]: Unimplemented op: %s.\n", dg_op_kind.c_str());
             res->state = SKIPPED;
-            res->reason = CASE_NOT_SUPPORTED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
@@ -422,7 +428,7 @@ void skip_unimplemented_graph_attribute(
         if (fpmath_mode != dnnl::fpmath_mode::strict
                 && fpmath_mode != dnnl::fpmath_mode::bf16) {
             res->state = SKIPPED;
-            res->reason = CASE_NOT_SUPPORTED;
+            res->reason = skip_reason::case_not_supported;
             return;
         }
     }
@@ -463,6 +469,19 @@ int doit(const prb_t *prb, res_t *res) {
         if (aop.kind_ == "End") { end_opid_v.emplace_back(aop.id_); }
     }
 
+    if (prb->expected_n_partition != 0) {
+        // If the expected partition num is specified by user with command line
+        // knob
+        if (partitions.size() != prb->expected_n_partition) {
+            BENCHDNN_PRINT(0,
+                    "Error: the expected number of partitions (%zu) doesn't "
+                    "coincide with the actual number of partitions returned "
+                    "(%zu).\n ",
+                    prb->expected_n_partition, partitions.size());
+            return res->state = FAILED, FAIL;
+        }
+    }
+
     if (partitions.empty()) {
         BENCHDNN_PRINT(0, "%s\n", "Error: partitions are empty");
         return res->state = FAILED, FAIL;
@@ -602,6 +621,7 @@ int doit(const prb_t *prb, res_t *res) {
         SAFE(ref_partition.init_ref(
                      graph_in_ports, partition_mem_map_v[i], res),
                 WARN);
+        if (res->state == SKIPPED) return OK;
 
         if (has_bench_mode_bit(mode_bit_t::corr)) {
             // correctness mode, run ref partition
@@ -648,19 +668,29 @@ int doit(const prb_t *prb, res_t *res) {
                     i);
             return res->state = FAILED, FAIL;
         }
-
         if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
 
         input_ts_all.emplace_back(input_ts);
         output_ts_all.emplace_back(output_ts);
 
+        auto &graph_mem_mgr = graph_mem_manager_t::get_instance();
+        graph_mem_mgr.start_graph_mem_check();
         BENCHDNN_PRINT(3, "[INFO]: Start execution of partition #%zd.\n", i);
-        c_partitions[i - idx_offset].execute(strm, input_ts, output_ts);
-        strm.wait();
+        // Need following clean-up steps as the memories have been mappped to
+        // device. Otherwise the deconstruction will fail.
+        DNN_GRAPH_SAFE(
+                c_partitions[i - idx_offset].execute(strm, input_ts, output_ts),
+                (WARN | NEED_CLEANUP), res);
+        DNN_GRAPH_SAFE(strm.wait(), WARN, res);
+        graph_mem_mgr.stop_graph_mem_check();
 
         // map memory from device back to host
         map_unmap_partition_mem(partition_mem_map_v[i], inputs, MAP, res);
         map_unmap_partition_mem(partition_mem_map_v[i], outputs, MAP, res);
+
+        // If the device is out-of-memory due to graph path execution, skip the
+        // case.
+        if (res->state == SKIPPED) return OK;
         if (res->state == FAIL) {
             BENCHDNN_PRINT(0,
                     "FAIL: Fail to map memories back to host for partition "
@@ -668,7 +698,6 @@ int doit(const prb_t *prb, res_t *res) {
                     i);
             return FAIL;
         }
-
         res->state = EXECUTED;
 
         if (has_bench_mode_bit(mode_bit_t::corr)) {
@@ -677,6 +706,15 @@ int doit(const prb_t *prb, res_t *res) {
                          partition_mem_map_v[i], res),
                     WARN);
         }
+
+        auto &graph_mem_req = graph_memory_req_args_t::get_instance();
+        // release the memory assigned for the reference path of the partition,
+        // while the memory for the graph path needs to be kept for the
+        // performance mode if needed.
+        graph_mem_req.reset_path(REF);
+        if (!has_bench_mode_bit(mode_bit_t::perf)) {
+            graph_mem_req.reset_path(GRAPH_USER);
+        }
     }
 
     if (has_bench_mode_bit(mode_bit_t::perf)) {
@@ -684,6 +722,7 @@ int doit(const prb_t *prb, res_t *res) {
                      input_ts_all, output_ts_all, res),
                 WARN);
     }
+
     return OK;
 }
 } // namespace graph
