@@ -425,7 +425,8 @@ void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
 
     if (is_gpu() && prb->attr.post_ops.len() != 0) {
         // GPU does not support post-ops
-        res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
+        res->state = SKIPPED;
+        res->reason = skip_reason::case_not_supported;
         return;
     }
 }
